(()=>{"use strict";var e,t={95:()=>{const e=window.React,t=window.wp.blocks,r=window.wp.i18n,l=window.wp.blockEditor,i=window.wp.data,s=window.wp.components,a=JSON.parse('{"UU":"siteseo/breadcrumbs"}');(0,t.registerBlockType)(a.UU,{edit:function({attributes:t}){let[a]=(0,i.useSelect)((e=>[e("core/editor").getEditedPostAttribute("title")||(0,r.__)("Example title","siteseo")]),[]),n=[];return t.prefix&&n.push({title:t.prefix,url:!1}),t.hideHome||n.push({title:t.homeLabel?t.homeLabel:(0,r.__)("Home"),url:!0}),n.push({title:a,url:!1}),(0,e.createElement)(e.Fragment,null,(0,e.createElement)(s.Notice,{status:"warning",isDismissible:!1,politeness:"polite"},(0,e.createElement)("p",null,(0,r.__)("This is just a preview, to see the final result, please view this page as a normal user","siteseo"))),(0,e.createElement)("div",{...(0,l.useBlockProps)()},(0,e.createElement)("ul",{className:"siteseo-breadcrumbs"},n.map(((r,l)=>(0,e.createElement)(e.Fragment,null,(0,e.createElement)("li",{key:r.url},r.url?(0,e.createElement)("a",{href:"#",title:r.title},r.title):r.title),(0,e.createElement)("div",{className:"siteseo-breadcrumbs-seperator"},(0,e.createElement)("span",null,n.length-1!=l?t.seperator:""))))))))},icon:function(){return(0,e.createElement)("svg",{xmlns:"http://www.w3.org/2000/svg",width:"664.82",height:"495.38",viewBox:"0 0 43.214 32.2"},(0,e.createElement)("path",{d:"M1 32.1c-.6 0-1-.4-1-1V1c0-.6.4-1 1-1h8.1c.4 0 .9.2 1.2.5l11.6 14.6c.4.5.4 1.3 0 1.9L10.2 31.6c-.3.4-.8.6-1.3.6zM42.914 15.071l-11.7-14.5c-.5-.6-1.4-.8-2.1-.2l-2.3 1.9c-.7.5-.8 1.5-.2 2.1l9.5 11.7-9.5 11.8c-.5.6-.4 1.6.2 2.1l2.3 1.9c.7.5 1.5.4 2.1-.2l11.7-14.6c.4-.8.4-1.5 0-2z"}),(0,e.createElement)("path",{d:"m32.407 15.071-11.7-14.5c-.5-.6-1.4-.8-2.1-.2l-2.3 1.9c-.7.5-.8 1.5-.2 2.1l9.5 11.7-9.5 11.8c-.5.6-.4 1.6.2 2.1l2.3 1.9c.7.5 1.5.4 2.1-.2l11.7-14.6c.4-.8.4-1.5 0-2z"}))}})}},r={};function l(e){var i=r[e];if(void 0!==i)return i.exports;var s=r[e]={exports:{}};return t[e](s,s.exports,l),s.exports}l.m=t,e=[],l.O=(t,r,i,s)=>{if(!r){var a=1/0;for(u=0;u<e.length;u++){for(var[r,i,s]=e[u],n=!0,o=0;o<r.length;o++)(!1&s||a>=s)&&Object.keys(l.O).every((e=>l.O[e](r[o])))?r.splice(o--,1):(n=!1,s<a&&(a=s));if(n){e.splice(u--,1);var c=i();void 0!==c&&(t=c)}}return t}s=s||0;for(var u=e.length;u>0&&e[u-1][2]>s;u--)e[u]=e[u-1];e[u]=[r,i,s]},l.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),(()=>{var e={57:0,350:0};l.O.j=t=>0===e[t];var t=(t,r)=>{var i,s,[a,n,o]=r,c=0;if(a.some((t=>0!==e[t]))){for(i in n)l.o(n,i)&&(l.m[i]=n[i]);if(o)var u=o(l)}for(t&&t(r);c<a.length;c++)s=a[c],l.o(e,s)&&e[s]&&e[s][0](),e[s]=0;return l.O(u)},r=globalThis.webpackChunkbreadcrumbs=globalThis.webpackChunkbreadcrumbs||[];r.forEach(t.bind(null,0)),r.push=t.bind(null,r.push.bind(r))})();var i=l.O(void 0,[350],(()=>l(95)));i=l.O(i)})();