.wp-block-comments-pagination-numbers a {
  text-decoration: underline;
}
.wp-block-comments-pagination-numbers .page-numbers {
  margin-left: 2px;
}
.wp-block-comments-pagination-numbers .page-numbers:last-child {
  margin-right: 0;
}