#siteseo-cookie-bar .siteseo-cookie-bar-content{
display: flex;
flex-direction: column;
align-items: center;
gap: 5px;
}

#siteseo-cookie-bar{
position: fixed;
z-index: 9999;
width: 100%;
padding: 15px 20px;
display: flex;
justify-content: center;
align-items: center;
box-shadow: 0 2px 5px rgba(0, 0, 0, 0.2);
text-align: center;
box-sizing: border-box;
display: none;
}

.siteseo-cookie-bar-bottom{
bottom: 0;
left: 0;
}

.siteseo-cookie-bar-middle{
top: 50%;
left: 50%;
transform: translate(-50%, -50%);
width: auto;
}

.siteseo-cookie-bar-top{
top: 0;
left: 0;
}

.siteseo-cookie-bar-backdrop{
position: fixed;
top: 0;
left: 0;
width: 100%;
height: 100%;
background: rgba(0,0,0,0.5);
z-index: 9998;
display: none;
}

.siteseo-cookie-bar-button{
padding: 10px 20px;
border: none;
border-radius: 5px;
cursor: pointer;
font-size: 14px;
margin: 5px;
}

#siteseo-cookie-bar-accept{
background-color: var(--primary-btn-bg, #0073aa);
color: var(--primary-btn-text, #ffffff);
}

#siteseo-cookie-bar-accept:hover{
background-color: var(--primary-btn-hover-bg, #005f8d);
color: var(--primary-btn-hover-text, #ffffff);
}

#siteseo-cookie-bar-close{
background-color: var(--secondary-btn-bg, #e0e0e0);
color: var(--secondary-btn-text, #000000);
}

#siteseo-cookie-bar-close:hover{
background-color: var(--secondary-btn-hover-bg, #c4c4c4);
color: var(--secondary-btn-hover-text, #000000);
}

#siteseo-cookie-bar-manage-btn{
position: fixed;
bottom: 20px;
left: 20px;
z-index: 9999;
cursor: pointer;
padding: 10px 15px;
border-radius: 5px;
box-shadow: 0 2px 4px rgba(0, 0, 0, 0.1);
display: block;
text-decoration: none;
transition: background-color 0.3s ease;
}

#siteseo-cookie-bar-manage-btn:hover{
background-color: var(--primary-btn-hover-bg, #005f8d);
color: var(--primary-btn-hover-text, #ffffff);
}

.siteseo-cookie-bar-buttons{
display: flex;
justify-content: center;
align-items: center;
gap: 10px;
margin-top: 10px;
}