<div  class="pd-20 settings">

        <div class="form-group row">
            <label for="jetlicense" class="col-sm-3 col-form-label jb-label">{{ lang.t("License") }}</label>
            <div class="col-sm-9">
                <div class="pd-3 jobs">
                    <input type="text" autocomplete="off" placeholder="XXXXXXXXXXXXX" id="jetlicense" class="form-control" name="LICENSE_KEY"  ng-model="saveData.LICENSE_KEY" value=""/>
                    <p class="pd-3">{{ lang.t("Enter your backup plugin license key here to activate premium features, including enhanced backup options and priority support. Ensure your license key is valid and active to maintain uninterrupted access to all benefits.") }}</p>
                </div>
            </div>
        </div>

        <div class="form-group row">
            <label for="timezones" class="col-sm-3 col-form-label jb-label">{{ lang.t("Timezone") }}</label>
            <div class="col-sm-9">
                <div class="pd-3">
                    <select id="timezones" class="form-select" aria-label="Timezone" name="TIMEZONE" ng-model="saveData.TIMEZONE">
                        <option value="WORDPRESS_TIMEZONE">Use WordPress Settings</option>
                        <option ng-repeat="(zone, offset) in timezones" value="{{zone}}">
                            {{zone}} (GMT {{offset > 0 ? '+' : ''}}{{offset}})
                        </option>
                    </select>
                    <p class="pd-3">{{ lang.t("You can override the default WordPress timezone settings by choosing a specific timezone here.") }}</p>
                </div>
            </div>
        </div>

        <div class="form-group row">
            <label for="COMMUNITY_LANGUAGES" class="col-sm-3 col-form-label jb-label">{{ lang.t('Community Languages') }}</label>
            <div class="col-sm-9">
                <div class="pd-3 switch-main">
                    <label class="switch">
                        <input id="COMMUNITY_LANGUAGES"
                        type="checkbox"
                        ng-model="saveData.COMMUNITY_LANGUAGES"
                        ng-true-value=1
                        ng-false-value=0>
                        <span class="slider round"></span>
                        <input type="hidden" name="COMMUNITY_LANGUAGES" ng-value="saveData.COMMUNITY_LANGUAGES">
                    </label>
                    <p class="pd-3">
                        {{ lang.t('Enable/Disable community languages delivered by our languages') }}
                        <a href="https://localazy.com/p/jetbackup-wordpress" target="_blank">CDN</a>.
                    </p>
                </div>
            </div>
        </div>

        <div class="form-group row">
            <label for="JETBACKUP_INTEGRATION" class="col-sm-3 col-form-label jb-label">{{ lang.t('JetBackup Server Integration') }}</label>
            <div class="col-sm-9">
                <p class="pd-3 text-danger" ng-if="is_windows">{{ lang.t('Not relevant for Windows OS') }}</p>
                <p class="pd-3 text-danger" ng-if="!is_windows && !is_jetbackup_linux && !saveData.JETBACKUP_SOCKET_API_ERROR_MESSAGE">
                   {{ lang.t('JetBackup server integration is not available on this system.') }}
                </p>
                <p class="pd-3 text-danger" ng-if="!is_windows && !is_jetbackup_linux && saveData.JETBACKUP_SOCKET_API_ERROR_MESSAGE">
                   {{ saveData.JETBACKUP_SOCKET_API_ERROR_MESSAGE }}
                </p>
                <div class="pd-3 switch-main" ng-if="!is_windows && is_jetbackup_linux">
                    <label class="switch">
                        <input id="JETBACKUP_INTEGRATION"
                                type="checkbox"
                                ng-model="saveData.JETBACKUP_INTEGRATION"
                                ng-true-value=1
                                ng-false-value=0
                                ng-disabled="!is_jetbackup_linux || !!!saveData.JETBACKUP_SOCKET_API_STATUS">
                        <span class="slider round"></span>
                        <input type="hidden" name="JETBACKUP_INTEGRATION" ng-value="saveData.JETBACKUP_INTEGRATION">
                    </label>
                    <p class="pd-3">{{ lang.t('Enable/Disable API to server level JetBackup (Restore backups generated by your hosting provider\'s JetBackup).') }}</p>
                    <p class="pd-3 text-danger" ng-if="is_jetbackup_linux && saveData.JETBACKUP_SOCKET_API_STATUS != 1">
                        * {{ lang.t('JetBackup socket API is not enabled or not responding on this system') }}<span ng-if="saveData.JETBACKUP_SOCKET_API_ERROR_MESSAGE"> [ {{saveData.JETBACKUP_SOCKET_API_ERROR_MESSAGE}} ] </span>*
                    </p>
                </div>
            </div>
        </div>

    <div class="form-group row">
        <label for="ADMIN_TOP_MENU_INTEGRATION" class="col-sm-3 col-form-label jb-label">{{ lang.t('Admin Top Menu Bar Integration') }}</label>
        <div class="col-sm-9">

            <div class="pd-3 switch-main">
                <label class="switch">
                    <input id="ADMIN_TOP_MENU_INTEGRATION"
                           type="checkbox"
                           ng-model="saveData.ADMIN_TOP_MENU_INTEGRATION"
                           ng-true-value=1
                           ng-false-value=0>
                    <span class="slider round"></span>
                    <input type="hidden" name="ADMIN_TOP_MENU_INTEGRATION" ng-value="saveData.ADMIN_TOP_MENU_INTEGRATION">
                </label>

                <p class="pd-3">{{ lang.t('Enable/Disable Admin top menu bar integration.') }}</p>
            </div>
        </div>
    </div>

    <div class="form-group row">
        <label for="DISPLAY_LOCAL_FREE_DISK_SPACE" class="col-sm-3 col-form-label jb-label">{{ lang.t('Show available disk space (System info)') }}</label>
        <div class="col-sm-9">

            <div class="pd-3 switch-main">
                <label class="switch">
                    <input id="DISPLAY_LOCAL_FREE_DISK_SPACE"
                           type="checkbox"
                           ng-model="saveData.DISPLAY_LOCAL_FREE_DISK_SPACE"
                           ng-true-value=1
                           ng-false-value=0>
                    <span class="slider round"></span>
                    <input type="hidden" name="DISPLAY_LOCAL_FREE_DISK_SPACE" ng-value="saveData.DISPLAY_LOCAL_FREE_DISK_SPACE">
                </label>

                <p class="pd-3">{{ lang.t('Enable this to display available disk space in the “System info” page. Note: this value is usually taken from the server’s filesystem and may not reflect your account’s quota, so it can appear much higher than what you can actually use (e.g., 100TB shown vs 10GB quota).') }}</p>
            </div>
        </div>
    </div>

        <div class="form-group row">
            <label for="ManualRetention" class="col-sm-3 col-form-label jb-label">{{ lang.t('Manual Backups Retention') }}</label>
            <div class="col-sm-9">
                <div class="pd-3 jobs">
                    <input type="number" id="ManualRetention" class="form-control" name="MANUAL_BACKUPS_RETENTION"
                           ng-model="saveData.MANUAL_BACKUPS_RETENTION"/>
                    <p class="pd-3">{{ lang.t('This setting determines how many manual backups to keep') }} <strong>{{ lang.t('per destination') }}</strong>. {{ lang.t('Set to 0 to disable retention and never delete backups.') }}</p>
                </div>
            </div>
        </div>

        <div class="form-group row">
            <label for="ImportedRetention" class="col-sm-3 col-form-label jb-label">{{ lang.t('Imported Backups Retention') }}</label>
            <div class="col-sm-9">
                <div class="pd-3 jobs">
                    <input type="number" id="ImportedRetention" class="form-control" name="IMPORTED_BACKUPS_RETENTION"
                           ng-model="saveData.IMPORTED_BACKUPS_RETENTION"/>
                    <p class="pd-3">{{ lang.t('This setting determines how many imported backups to keep. Set to 0 to disable retention and never delete imported backups.') }}</p>
                </div>
            </div>
        </div>

        <div class="form-group row">
            <label for="php_cli_binary" class="col-sm-3 col-form-label jb-label">{{ lang.t('PHP CLI Binary Location') }}</label>
            <div class="col-sm-9">
                <div class="pd-3 jobs">
                    <input type="text" placeholder="/opt/cpanel/ea-php82/root/bin/php" id="php_cli_binary" class="form-control" name="PHP_CLI_LOCATION"
                           ng-model="saveData.PHP_CLI_LOCATION"/>
                    <p class="pd-3">{{ lang.t('JetBackup relies on PHP CLI for background operations, here you can set specific path for php binary.') }}</p>
                </div>
            </div>
        </div>

        <div class="form-group row">
            <label for="mysql_default_port" class="col-sm-3 col-form-label jb-label">{{ lang.t('MySQL Default Port') }}</label>
            <div class="col-sm-9">
                <div class="pd-3 jobs">
                    <input type="number" placeholder="3306" id="mysql_default_port" class="form-control" name="MYSQL_DEFAULT_PORT"
                           ng-model="saveData.MYSQL_DEFAULT_PORT"/>
                    <p class="pd-3">
                        {{ lang.t('JetBackup will attempt to automatically detect the MySQL port from your system settings. If it cannot be determined, this value will be used as the default fallback.') }}
                    </p>
                </div>
            </div>
        </div>

        <div class="form-group row">
            <label for="alternate_wp_config_location" class="col-sm-3 col-form-label jb-label">{{ lang.t("Alternate 'wp-config.php' Location") }}</label>
            <div class="col-sm-9">
                <div class="pd-3 jobs">
                    <input type="text" placeholder="/home/user/wp-config-folder/alternate-wp-config.php" id="alternate_wp_config_location" class="form-control" name="ALTERNATE_WP_CONFIG_LOCATION"
                           ng-model="saveData.ALTERNATE_WP_CONFIG_LOCATION"/>
                    <p class="pd-3">{{ lang.t('Use this if your hosting provider uses a non-default wp-config location') }}</p>
                </div>
            </div>
        </div>

</div>