@media screen and (max-width: 700px) {

    header {
        position: fixed;
        top: 0;
        left: 0;
        width: 100%;
        z-index: 1002;
        background-image: url("../images/logo-small.png");
        background-position: 10px 50%;
        background-repeat: no-repeat;
    }

    #logo {
        display: none;
    }

    #menu {
        position: fixed;
        width: 100%;
        z-index: 1000;
        top: 80px;
        box-shadow: none;
    }

    .menu-closed #menu {
        width: 0;
    }

    .menu-closed ul#navigation li a {
        border-radius: 10px;
    }

    .menu-closed ul#navigation li a i {
        font-size: 16px;
        width: 22.5px;
    }

    #container {
        margin-left: 0 !important;
        margin-top: 80px;
    }

    #menu-bars {
        display: block;
        color: var(--jb-dark-blue);
    }

    .modal.left .modal-dialog,
    .modal.right .modal-dialog {
        width: 100%;
        max-width: 100%;
    }

    .modal.left.fade .modal-dialog,
    .modal.right.fade .modal-dialog {
        left: 0;
    }
}