<?php
/*
*
* JetBackup @ package
* Created By Idan Ben-Ezra
*
* Copyrights @ JetApps
* https://www.jetapps.com
*
**/
namespace JetBackup\Destination\Vendors\S3\Client\Exception;

defined( '__JETBACKUP__' ) or die( 'Restricted access' );

class ClientException extends \Exception {
}