.siteseo-navbar{
margin: 0;
display: flex;
align-items: center;
justify-content: space-between;
padding: 10px 20px;
background-color: white;
border-bottom: 1px solid #e0e0e0;
position: sticky; 
top: 32px; 
width: 100%;
z-index: 1000; 
}

.siteseo-navbar .logo{
display: flex;
align-items: center;
}

.siteseo-navbar .logo img{
height: 30px;
}

.siteseo-navbar .siteseo-breadcrumb{
display: flex;
align-items: center;
margin-left: 10px;
}

.siteseo-navbar .siteseo-breadcrumb a{
text-decoration: none;
font-weight: bold;
color: grey;
font-size: 13px;
}

.siteseo-navbar .siteseo-breadcrumb a.active{
font-weight: bold;
color: black;
}

.siteseo-navbar .siteseo-breadcrumb span{
margin: 0 5px;
color: grey;
}

.siteseo-navbar .links{
display: flex;
align-items: center;
gap:20px;
}

.siteseo-navbar .links a{
text-decoration: none;
color: #0033a0;
font-size: 12px;
font-weight: bold;
}

.siteseo-navbar .links .support{
background-color: #0033a0;
color: white;
font-weight: bold;
padding: 5px 15px;
border-radius: 5px;
height: 25px;
display: flex;
align-items: center;
justify-content: center;
}

/* dash line */
.siteseo-dashline{
border-top: 1px dashed #e0e0e0;
width: 100%;
}

/* tooggle btn */
.siteseo-toggle-cnt{
display: flex;
align-items: center;
}
    
.siteseo-toggle-Sw{
position: relative;
width: 47px;
height: 20px;
background-color: #ccc;
border-radius: 20px;
cursor: pointer;
margin-right: 10px;
transition: background-color 0.3s;
}

.siteseo-toggle-Sw::before{
content: '';
position: absolute;
width: 24px;
height: 24px;
background-color: white;
border: 2px solid #ccc;
border-radius: 50%;
top: -4px;
left: -2px;
transition: 0.3s;
}

.row.mx-0 .siteseo-toggle-Sw::before{
top: -2px;
}

.siteseo-toggle-cnt span{
font-size: 14px;
color: #555;
font-weight: bold;
}

.siteseo-toggle-Sw.active{
background-color: #003399;
}

.siteseo-toggle-Sw.active::before{
left: 27px;
border-color: #003399;
}

.siteseo-arrow{
margin-right: 5px;
font-size: 14px;
color: #555;
font-weight: bold;
}

.siteseo-toggle-cnt p{
font-style: italic;
font-size: 11px;
font-weight: normal;
}

.siteseo-arrow-icon{
font-size: 10%;
margin-top:7px;
margin-right:3px;
}

.siteseo-option{
background-color: white;
}

/* sitseo btn */
.siteseo-button{
background-color: #003399;
color: white;
padding: 15px 30px;
border: none;
border-radius: 5px;
font-size: 16px;
cursor: pointer;
}

.top-line{
width: 100%;
height: 1px;
background-color: #d1d1d1;
margin-bottom: 2%;
}

/** toogle for post and pages */    
.siteseo-toggle-meta{
position: relative;
width: 47px;
height: 20px;
background-color: #ccc;
border-radius: 20px;
cursor: pointer;
margin-right: 10px;
transition: background-color 0.3s;
}

.siteseo-toggle-meta::before{
content: '';
position: absolute;
width: 24px;
height: 24px;
background-color: white;
border: 2px solid #ccc;
border-radius: 50%;
top: -4px;
left: -2px;
transition: 0.3s;
}

.row.mx-0 .siteseo-toggle-meta::before{
top : -2px;
}

.siteseo-toggle-meta.active{
background-color: #003399;
}

.siteseo-toggle-meta.active::before{
left: 27px;
border-color: #003399;
}

.siteseo-header-version-badge{
padding: 3px 10px;
border-radius:6px;
background-color:#f1f1f191;
font-weight:500;
}