<div class="modal-header">
	<div class="modal-main-title">
		<h1 class="modal-title fs-5" id="extractBackupLabel">{{ lang.t("Viewing Log") }}</h1>
	</div>
	<button type="button" class="btn-close" ng-click="$dismiss()" aria-label="{{ lang.t('Close') }}"></button>
</div>

<div class="modal-body">

	<div ng-if="loading" class="text-center my-4">
		<div class="jb-loading-screen">
			<p class="jb-loading-text">
				Loading<span class="jb-dots"><span></span><span></span><span></span></span>
			</p>
		</div>
	</div>

	<div ng-if="!loading" class="mt-3">
		<div class="row gx-2 align-items-start">
			<div class="col-auto">
				<label for="logPath" class="form-label mb-0">{{ lang.t("Path") }}: </label>
			</div>
			<div class="logPath col-md-10" id="logPath">
				{{ path }}
			</div>
		</div>
		<div class="row gx-2 align-items-start mt-2">
			<div class="col-auto">
				<label for="logSize" class="form-label mb-0">{{ lang.t("Size") }}: </label>
			</div>
			<div class="logPath col-md-3" id="logSize">
				{{ size }}
			</div>
		</div>
	</div>




	<div ng-if="!loading"  class="form-group mt-4">
		<label for="logContent" class="form-label">{{ lang.t("Log content") }}:</label>
			<div class="input-group">
				<textarea readonly id="logContent"
						  class="form-control"
						  rows="6"
				>{{content}}</textarea>
			</div>
		</div>

	</div>

<div class="modal-footer">
	<button class="btn btn-secondary cancel_btn" type="button" ng-click="$dismiss()">
		{{ lang.t("Close") }}
	</button>
</div>
