<?php
/*
*
* JetBackup @ package
* Created By Idan Ben-Ezra
*
* Copyrights @ JetApps
* https://www.jetapps.com
*
**/
namespace JetBackup\Exception;

if (!defined( '__JETBACKUP__')) die('Direct access is not allowed');

class ArchiveException extends JBException {
}
