<h1>JetBackup Status Update</h1>
<p>Hello,</p>
<p>This is a notification regarding the scheduled backup job <strong>"{$job_name}"</strong>.</p>

<p>We are set to notify if the job was not executed for {$job_monitor} days. </p>
<p>It appears this interval has now passed.</p>

<p><strong>Domain:</strong> {$backup_domain}</p>
<p>Last backup job run date: {$backup_date}</p>

<p>Best regards,</p>
<p>JetBackup For WordPress</p>

