<div ng-controller="destinationGoogleDrive">

   <div class="form-group row" ng-if="!saveData.options.refresh_token">
      <label for="gdrive_access_code" class="col-sm-3 col-form-label jb-label">Access Code</label>
      <div class="col-sm-7">
         <input id="gdrive_access_code" class="form-control col-md-7 col-xs-12 jb-input jb-grey-0" type="text" ng-model="saveData.options.access_code" />
         <div class="d-flex justify-content-start mt-2">
            <span class="input-group-btn">
                <button id="gdrive_request_access_code" type="button" class="btn btn-primary ng-binding jb-background-orange border-0" ng-click="openAuthentication()">Request Access Code</button>
            </span>
         </div>
         <p class="pd-3">Click to request an access code. The code will appear on your screen. Please copy it to your clipboard and paste it in the required field.</p>
      </div>
   </div>

   <div class="form-group row">
      <label for="gdrive_retries" class="col-sm-3 col-form-label jb-label">{{ lang.t("HTTP Retries") }}</label>
      <div class="col-sm-7">
         <input-dropdown addon="Retries" options="{0:lang.t('Disabled'),1:1,3:3,5:5,10:10}" ng-model="saveData.options.retries">
            <input id="gdrive_retries" validate-field="int" min="0" class="form-control" ng-model="saveData.options.retries" id="retries" type="text" value="" />
         </input-dropdown>
         <p class="pd-3">{{ lang.t("If the HTTP call (upload / download / delete) fails for some reason retry for x times. Maximum 10") }}</p>
      </div>
   </div>

</div>