<?php
/*!
* Hybridauth
* https://hybridauth.github.io | https://github.com/hybridauth/hybridauth
*  (c) 2017 Hybridauth authors | https://hybridauth.github.io/license.html
*/

namespace Hybridauth\Exception;

/**
 *
 */
class InvalidOpenidIdentifierException extends InvalidArgumentException implements ExceptionInterface
{
}
