.lz_pro_firewall_save_btn{
display: flex;
justify-content: center;
align-items: center;
}

#lz_pro_country_container{
/* display:none; */
width:80%;
max-height:300px;
overflow-y:auto;
border:1px solid #ccc;
border-radius:5px;
}

#lz_pro_country_container li{
border-bottom: .5px solid #f8f4f4;
margin-bottom: 0px;
padding: 8px 6px;
font-size:medium;
font-weight:500;
cursor:pointer;
}

#lz_pro_country_container li:hover{
background-color: #e4ecec;
}

.lz_pro_blocked_country_container{
display: flex;
flex-wrap: wrap;
gap: 10px;
}

.lz_pro_blocked_country_container div{
display: flex;
gap: 4px;
border: 1px solid rgb(228, 215, 215);
padding: 5px 10px;
border-radius:50px;
}

.lz_pro_remove_country:hover{
color: #2271b1;
cursor: pointer;
}

.lz-country-db-status-wrapper{
display:flex;
background-color:#eff6ff;
gap:20px;
border-radius:6px;
border:1px solid #dbeafe;
padding:1.25rem;
max-width:80%;
}

.lz-country-db-status-icon-wrap{
background-color:#dbeafe;
padding: 10px;
border-radius:50px;
display:inline-block;
height:20px;
width:20px;	
}

.lz-country-db-status-icon-wrap dashicons{
color:#3b82f6;
}

.lz-country-db-status-main{
width:100%;
}

.lz-country-db-status-main-content{
display:flex;
justify-content:space-between;
}

.lz-country-db-status-main-content h3{
margin:0;
text-transform:uppercase;
letter-spacing:0.025em;
font-size: 0.875rem;
line-height: 1.25rem;
}

.lz-country-db-status-main-content p{
margin:0;
font-weight:400;
color:#374151;
font-size: 0.875rem;
line-height: 1.25rem;
}

.lz-country-block-db-state{
height: 0.625rem;
width: 0.625rem;
border-radius:9999px;
display:inline-block;
margin-right:5px;
}

.lz-country-block-download-notice{
padding:1rem;
background-color:white;
border:1px solid #dbeafe;
border-radius:6px;
}

.lz-country-block-download-notice .dashicons-update{
color:#3b82f6;
animation: lz-animate-spin 1s linear infinite;
}

@keyframes lz-animate-spin{
100%{
transform: rotate(360deg);
}
}

/* Start firewall configuration popup */
.loginizer-htaccess-popup-modal{
position: fixed;
display: flex;
align-items: center;
justify-content: center;
top: 0;
left: 0;
width: 100%;
height: 100%;
background-color: rgba(0, 0, 0, .7);
z-index: 9999;
}

.loginizer-htaccess-popup-box{
display: flex;
flex-direction: column;
width: 400px;
background: #ffffff;
color: #1f1f1f;
padding: 20px;
border-radius: 10px;
box-shadow: 0 10px 25px rgba(0, 0, 0, 0.2);
}

.loginizer-htaccess-popup-header{
display: flex;
justify-content: space-between;
padding-bottom: 5px;
border-bottom: 1px solid #c2c2c2;
}

.loginizer-popup-heading{
font-size: 18px;
font-weight: 500;
}

.loginizer-file-path{
background: #f9fafb;
border: 1px solid #e5e7eb;
padding: 8px 10px;
margin-top: 4px;
border-radius: 6px;
word-break: break-all;
}

.loginizer-htaccess-popup-select{
width: 100%;
padding: 8px 10px;
border-radius: 6px;
border: 1px solid #d1d5db;
}

.loginizer-htaccess-popup-footer{
display: flex;
flex-wrap: wrap;
gap: 5px;
}

.loginizer_htaccess_popup_submit_button{
margin-top: 10px;
padding-top: 12.5px;
display: flex;
justify-content: center;
border-top: 1px Solid #ccc;
}

.loginizer-htaccess-popup-error{
	margin-top: 5px;
	padding: 10px;
	color: #ff0000;
	font-weight: 400;
	border : 0.5px solid #c2c2c2;
	border-left: 4px solid #ff0000;
}

#lz_pro_configure_htaccess:disabled{
	opacity: 0.8;
	cursor: none;
}
/* End firewall configuration popup */

.lz-cb-error-table th{
	background:#EFEFEF;
	position:sticky;
	top:32px;
	z-index:10;
}
