<?php

namespace WordPress\AiClientDependencies\Http\Discovery\Exception;

/**
 * Thrown when we can't use Puli for discovery.
 *
 * @author Tobias Nyholm <tobias.nyholm@gmail.com>
 */
final class PuliUnavailableException extends StrategyUnavailableException
{
}
