<?php

namespace CookieAdmin;

if(!defined('COOKIEADMIN_VERSION') || !defined('ABSPATH')){
	die('Hacking Attempt');
}

$cookieadmin_policies = array (
  'cookieadmin_gdpr' => 
  array (
	'cookieadmin_layout' => 'box',
	'cookieadmin_modal' => 'center',
	'cookieadmin_position' => 'bottom_left',
	'cookieadmin_days' => '365',
	'cookieadmin_notice_title' => __('We respect your privacy', 'cookieadmin'),
	'cookieadmin_notice' => __('Cookies help us improve your experience, deliver personalized content, and analyze traffic. You can choose which cookies to allow by clicking <b>Customize</b>. Click <b>Accept All</b> to consent or <b>Reject All</b> to decline non-essential cookies.', 'cookieadmin'),
	'cookieadmin_preference_title' => __('Personalize Your Cookie Preferences', 'cookieadmin'),
	'cookieadmin_preference' => __('We use cookies to ensure smooth navigation and enable essential site functions. You can view detailed information about each cookie category below. <br />Cookies marked as <b>Necessary</b> are stored in your browser because they are essential for basic site functionality. <b>These cookies do not require your consent under GDPR.</b> <br />We also use third-party cookies to analyze site usage, remember your preferences, and deliver relevant content and ads. These will only be activated with your consent. You can choose to enable or disable these cookies, but please note that turning off some types may affect your browsing experience.', 'cookieadmin'),
	'reConsent_title' => __('Modify Cookie Preferences', 'cookieadmin'),
	'cookieadmin_notice_title_color' => '#000000',
	'cookieadmin_notice_color' => '#000000',
	'cookieadmin_consent_inside_bg_color' => '#ffffff',
	'cookieadmin_consent_inside_border_color' => '#abb8c3',
	'cookieadmin_preference_title_color' => '#000000',
	'cookieadmin_details_wrapper_color' => '#000000',
	'cookieadmin_cookie_modal_bg_color' => '#ffffff',
	'cookieadmin_cookie_modal_border_color' => '#000000',
	'cookieadmin_customize_btn' => __('Customize', 'cookieadmin'),
	'cookieadmin_customize_btn_color' => '#000000',
	'cookieadmin_customize_btn_bg_color' => '#ffffff',
	'cookieadmin_reject_btn' => __('Reject All', 'cookieadmin'),
	'cookieadmin_reject_btn_color' => '#000000',
	'cookieadmin_reject_btn_bg_color' => '#ffffff',
	'cookieadmin_accept_btn' => __('Accept All', 'cookieadmin'),
	'cookieadmin_accept_btn_color' => '#ffffff',
	'cookieadmin_accept_btn_bg_color' => '#1863dc',
	'cookieadmin_save_btn' => __('Save Preferences', 'cookieadmin'),
	'cookieadmin_save_btn_color' => '#ffffff',
	'cookieadmin_save_btn_bg_color' => '#1863dc',
	'cookieadmin_slider_off_bg_color' => '#808080',
	'cookieadmin_slider_on_bg_color' => '#3582C4',
	'cookieadmin_links_color' => '#1863dc',
  ),
  'cookieadmin_us' => 
  array (
	'cookieadmin_layout' => 'box',
	'cookieadmin_modal' => 'center',
	'cookieadmin_position' => 'bottom_left',
	'cookieadmin_days' => '365',
	'cookieadmin_notice_title' => __('Your Privacy Matters', 'cookieadmin'),
	'cookieadmin_notice' => __('We use cookies to improve your browsing experience, deliver personalized content and ads, and analyze website traffic. By clicking <b>Accept All</b>, you consent to our use of cookies. You can choose to manage your preferences or opt out of the sale or sharing of your personal information.', 'cookieadmin'),
	'cookieadmin_preference_title' => __('Customize Your Cookie Settings', 'cookieadmin'),
	'cookieadmin_preference' => __('We use cookies to support essential site functions and help you navigate efficiently. You can find detailed information about each cookie category below. <br /><b>Necessary</b> cookies are required for core site functionality and are always enabled. These do not require your consent. <br />Other cookies, including third-party cookies, are used to analyze usage, remember preferences, and provide relevant ads and content. These cookies are only used if you choose to enable them. You can adjust your preferences at any time. Disabling certain cookies may impact your browsing experience.', 'cookieadmin'),
	'reConsent_title' => __('Modify Cookie Preferences', 'cookieadmin'),
	'cookieadmin_notice_title_color' => '#000000',
	'cookieadmin_notice_color' => '#000000',
	'cookieadmin_consent_inside_bg_color' => '#ffffff',
	'cookieadmin_consent_inside_border_color' => '#abb8c3',
	'cookieadmin_preference_title_color' => '#000000',
	'cookieadmin_details_wrapper_color' => '#000000',
	'cookieadmin_cookie_modal_bg_color' => '#ffffff',
	'cookieadmin_cookie_modal_border_color' => '#000000',
	'cookieadmin_customize_btn' => __('Customize', 'cookieadmin'),
	'cookieadmin_customize_btn_color' => '#000000',
	'cookieadmin_customize_btn_bg_color' => '#ffffff',
	'cookieadmin_reject_btn' => __('Reject All', 'cookieadmin'),
	'cookieadmin_reject_btn_color' => '#000000',
	'cookieadmin_reject_btn_bg_color' => '#ffffff',
	'cookieadmin_accept_btn' => __('Accept All', 'cookieadmin'),
	'cookieadmin_accept_btn_color' => '#ffffff',
	'cookieadmin_accept_btn_bg_color' => '#1863dc',
	'cookieadmin_save_btn' => __('Save Preferences', 'cookieadmin'),
	'cookieadmin_save_btn_color' => '#ffffff',
	'cookieadmin_save_btn_bg_color' => '#1863dc',
	'cookieadmin_slider_off_bg_color' => '#808080',
	'cookieadmin_slider_on_bg_color' => '#3582C4',
	'cookieadmin_links_color' => '#1863dc',
  ),
);