/*
Pagelayer Frontend Framework
*/

[pagelayer-id]{
transition:0.5s;
}

/* Experimental All FLEX code 
.pagelayer-ele,
.pagelayer-ele-wrap,*/

/*Flex - Rows and Cols*/ 
.pagelayer-row,
.pagelayer-inner_row,
.pagelayer-row-holder,
.pagelayer-col,
.pagelayer-col-holder{
box-sizing: border-box;
display: flex;
flex: 1 0 auto;
flex-direction: row;
flex-wrap: wrap;
width:100%;
align-content: stretch;
position: relative;
}

/* The col holder should be flex-start and not stretch because elements in it should take automatic height */
.pagelayer-col,
.pagelayer-col-holder{
align-content: flex-start;
}

/* Each immediate element in the col-holder should have full width */ 
.pagelayer-col-holder>*{
width: 100%;
}

.pagelayer-row.pagelayer-auto .pagelayer-col {
flex-grow: 1; }

.pagelayer-col-1 {
  width: 8.33333%; }

.pagelayer-offset-1 {
  margin-left: 8.33333%; }

.pagelayer-col-2 {
  width: 16.66667%; }

.pagelayer-offset-2 {
  margin-left: 16.66667%; }

.pagelayer-col-3 {
  width: 25%; }

.pagelayer-offset-3 {
  margin-left: 25%; }

.pagelayer-col-4 {
  width: 33.33333%; }

.pagelayer-offset-4 {
  margin-left: 33.33333%; }

.pagelayer-col-5 {
  width: 41.66667%; }

.pagelayer-offset-5 {
  margin-left: 41.66667%; }

.pagelayer-col-6 {
  width: 50%; }

.pagelayer-offset-6 {
  margin-left: 50%; }

.pagelayer-col-7 {
  width: 58.33333%; }

.pagelayer-offset-7 {
  margin-left: 58.33333%; }

.pagelayer-col-8 {
  width: 66.66667%; }

.pagelayer-offset-8 {
  margin-left: 66.66667%; }

.pagelayer-col-9 {
  width: 75%; }

.pagelayer-offset-9 {
  margin-left: 75%; }

.pagelayer-col-10 {
  width: 83.33333%; }

.pagelayer-offset-10 {
  margin-left: 83.33333%; }

.pagelayer-col-11 {
  width: 91.66667%; }

.pagelayer-offset-11 {
  margin-left: 91.66667%; }

.pagelayer-col-12 {
  width: 100%; }

.pagelayer-offset-12 {
  margin-left: 100%; }

.pagelayer-gutters > .pagelayer-col-1 {
  width: calc(8.33333% - 2%); }

.pagelayer-gutters > .pagelayer-offset-1 {
  margin-left: calc(8.33333% + 2%) !important; }

.pagelayer-gutters > .pagelayer-col-2 {
  width: calc(16.66667% - 2%); }

.pagelayer-gutters > .pagelayer-offset-2 {
  margin-left: calc(16.66667% + 2%) !important; }

.pagelayer-gutters > .pagelayer-col-3 {
  width: calc(25% - 2%); }

.pagelayer-gutters > .pagelayer-offset-3 {
  margin-left: calc(25% + 2%) !important; }

.pagelayer-gutters > .pagelayer-col-4 {
  width: calc(33.33333% - 2%); }

.pagelayer-gutters > .pagelayer-offset-4 {
  margin-left: calc(33.33333% + 2%) !important; }

.pagelayer-gutters > .pagelayer-col-5 {
  width: calc(41.66667% - 2%); }

.pagelayer-gutters > .pagelayer-offset-5 {
  margin-left: calc(41.66667% + 2%) !important; }

.pagelayer-gutters > .pagelayer-col-6 {
  width: calc(50% - 2%); }

.pagelayer-gutters > .pagelayer-offset-6 {
  margin-left: calc(50% + 2%) !important; }

.pagelayer-gutters > .pagelayer-col-7 {
  width: calc(58.33333% - 2%); }

.pagelayer-gutters > .pagelayer-offset-7 {
  margin-left: calc(58.33333% + 2%) !important; }

.pagelayer-gutters > .pagelayer-col-8 {
  width: calc(66.66667% - 2%); }

.pagelayer-gutters > .pagelayer-offset-8 {
  margin-left: calc(66.66667% + 2%) !important; }

.pagelayer-gutters > .pagelayer-col-9 {
  width: calc(75% - 2%); }

.pagelayer-gutters > .pagelayer-offset-9 {
  margin-left: calc(75% + 2%) !important; }

.pagelayer-gutters > .pagelayer-col-10 {
  width: calc(83.33333% - 2%); }

.pagelayer-gutters > .pagelayer-offset-10 {
  margin-left: calc(83.33333% + 2%) !important; }

.pagelayer-gutters > .pagelayer-col-11 {
  width: calc(91.66667% - 2%); }

.pagelayer-gutters > .pagelayer-offset-11 {
  margin-left: calc(91.66667% + 2%) !important; }

.pagelayer-gutters > .pagelayer-col-12 {
  width: calc(100% - 2%); }

.pagelayer-gutters > .pagelayer-offset-12 {
  margin-left: calc(100% + 2%) !important; }

.pagelayer-first {
  order: -1; }

.pagelayer-last {
  order: 1; }

/**************My Style for front-end *********/
.pagelayer-img{
vertical-align:bottom;
max-width: 100%;
}

/* To give no decoration to a link for a pagelayer element */
.pagelayer-ele-link{
text-decoration:none;
box-shadow:none !important;
border:none;
}

.pagelayer-bgimg-slider{
position: absolute;
top: 0;
left: 0;
height: 100%;
width: 100%;
}

.pagelayer-bgimg-slide {
position: absolute;
width: 100%;
top: 0;
left: 0;
height: 100%;
transition: all 1s ease-in-out;
background-position: center center;
background-repeat: no-repeat;
background-size: cover;
opacity: 0;
}

.pagelayer-slide-show {
opacity: 1;
}

.pagelayer-row-overlay,
.pagelayer-col-overlay{
position: absolute;
width: 100%;
height: 100%;
right: 0px;
top: 0px;
z-index: -1;
pointer-events:none;
}

.pagelayer-row-shape{
width:100%;
position:absolute;
top:0;
left:0;
right:0;
bottom:0;
line-height:0;
overflow:hidden;
}

.pagelayer-row-svg{
position:relative;
height:100%;
}

.pagelayer-row-svg svg{
width:100%;
position:absolute;
}

.pagelayer-row-svg svg.pagelayer-svg-top{
top:-1px;
}

.pagelayer-row-svg svg.pagelayer-svg-bottom{
bottom:-1px;
}

.pagelayer-height-fit{
height:100vh;
}

.pagelayer-row-wrapper{
width:100%;
}

.pagelayer-background-overlay{
position: absolute;
width: 100%;
top: 0;
left:0;
height: 100%;
overflow: hidden;
}

.pagelayer-background-video{
position: absolute;
width: 100%;
top: 0;
height: 100%;
pointer-events: none;
z-index: 0;
overflow: hidden;
left:0;
}

.pagelayer-background-video iframe,
.pagelayer-background-video video{
position: absolute;
top: 50%;
left: 50%;
-webkit-transform: translateX(-50%) translateY(-50%);
-ms-transform: translateX(-50%) translateY(-50%);
transform: translateX(-50%) translateY(-50%);
max-width: none;
}

.pagelayer-service-container{
position:relative;
-webkit-box-align: start;
-webkit-align-items: flex-start;
-ms-flex-align: start;
align-items: flex-start;
}

.pagelayer-service-image{
line-height:0;
overflow:hidden;
}

.pagelayer-service-image img{
height: auto;
max-width: 100%;
}

.pagelayer-service-btn{
display:inline-block;
position:relative;
z-index:9;
}

.pagelayer-service-details{
width:100%;
}

.pagelayer-service-text{
word-break:break-word;
}

.pagelayer-service-align-left{
display:flex;
}

.pagelayer-service-align-right{
display:flex;
text-align: right;
-webkit-box-orient: horizontal;
-webkit-box-direction: reverse;
flex-direction: row-reverse;
}

.pagelayer-service-align-top{
display:block;
text-align: center;
}

.pagelayer-service-align-top .pagelayer-service-image{
margin:0 auto;
text-align:center;
}

.pagelayer-service-box-centered{
text-align: center;
}

.pagelayer-service-box-media-left,
.pagelayer-service-box-media-right{
display: flex;
}

.pagelayer-service-vertical-top{
align-items: flex-start;
-webkit-align-items: flex-start;
-webkit-box-align: start;
-ms-flex-align: start;
}

.pagelayer-service-vertical-middle{
align-items: center;
-webkit-align-items: center;
-webkit-box-align: center;
-ms-flex-align: center;
}

.pagelayer-service-vertical-bottom{
align-items: flex-end;
-webkit-align-items: flex-end;
-webkit-box-align: end;
-ms-flex-align: end;
}

.pagelayer-service-box-media-right .pagelayer-service-box-media-section{
order: 2;
}
.pagelayer-service-box-media-right .pagelayer-service-box-content-section{
order: 1;
}

.pagelayer-service-box-icon-holder{
display: inline-block;
color:#000000;
text-align: center;
line-height: 1;
font-size: 60px;
}

.pagelayer-service-icon{
line-height: 0;
}

.pagelayer-service-icon.pagelayer-service-framed i{
border:4px solid;
background-color:transparent !important;
}

.pagelayer-service-icon.pagelayer-service-stacked i,
.pagelayer-service-icon.pagelayer-service-framed i{
position:relative;
}

.pagelayer-service-icon.pagelayer-service-stacked i:before,
.pagelayer-service-icon.pagelayer-service-framed i:before{
position: absolute;
top: 50%;
left: 50%;
transform: translate(-50%, -50%);
}

.pagelayer-service-box-icon-holder.square-holder{ 
padding: 15px;
color:#000000; 
}

.pagelayer-service-box-icon-holder.circle-holder{ 
padding: 15px;
color:#000000;
border-radius:50% !Important; 
}

.pagelayer-service-box-icon-holder i{
position: relative;
display: block;
}

.pagelayer-service-heading{
line-height:1;
}

.pagelayer-box-link{
position: absolute;
top: 0;
left: 0;
width: 100%;
height: 100%;
}

/* Icon widget */

.pagelayer-icon-mini{
	font-size: 16px !important;
}

.pagelayer-icon-small{
	font-size: 24px !important;
}

.pagelayer-icon-large{
	font-size: 36px !important;
}

.pagelayer-icon-extra-large{
	font-size: 56px !important;
}

.pagelayer-icon-double-large{
	font-size: 78px !important;
}

.pagelayer-icon-circle,
.pagelayer-icon-outline-circle,
.pagelayer-social-shape-circle .pagelayer-icon-holder{
	border-radius: 50%;
}

.pagelayer-icon-rounded,
.pagelayer-icon-outline-rounded,
.pagelayer-social-shape-rounded .pagelayer-share-content,
.pagelayer-social-shape-rounded .pagelayer-icon-holder{
	border-radius: 10px;
}

/* Icon widget end */

/* Icon animation */
.pagelayer-animation-grow{
transition: transform 400ms;
}

.pagelayer-animation-grow:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-grow{
transform: scale(1.1);
}

.pagelayer-animation-shrink{
transition: transform 400ms;
}

.pagelayer-animation-shrink:hover,
.pagelayer-anim-par:hover .pagelayer-animation-shrink{
transform: scale(0.9);
}

@keyframes pagelayer-animation-pulse{25%{transform:scale(1.1)}75%{transform:scale(0.9)}}

.pagelayer-animation-pulse:hover,
.pagelayer-anim-par:hover .pagelayer-animation-pulse{
animation-name: pagelayer-animation-pulse;
animation-duration: 1s;
animation-timing-function: linear;
animation-iteration-count: infinite;
}

@keyframes pagelayer-animation-pulse-grow{to{transform:scale(1.1)}}

.pagelayer-animation-pulse-grow:hover,
.pagelayer-anim-par:hover .pagelayer-animation-pulse-grow{
animation-name: pagelayer-animation-pulse-grow;
animation-duration: 0.4s;
animation-timing-function: linear;
animation-iteration-count: infinite;
animation-direction:alternate;
}

@keyframes pagelayer-animation-pulse-shrink{to{transform:scale(0.9)}}

.pagelayer-animation-pulse-shrink:hover,
.pagelayer-anim-par:hover .pagelayer-animation-pulse-shrink{
animation-name: pagelayer-animation-pulse-shrink;
animation-duration: 0.4s;
animation-timing-function: linear;
animation-iteration-count: infinite;
animation-direction:alternate;
}

@keyframes pagelayer-animation-push{50%{transform:scale(0.8)}100%{transform:scale(1)}}

.pagelayer-animation-push:hover,
.pagelayer-anim-par:hover .pagelayer-animation-push{
animation-name:pagelayer-animation-push;
animation-duration:0.4s;
animation-timing-function:linear;
animation-iteration-count:1
}

@keyframes pagelayer-animation-pop{50%{transform:scale(1.2)}}

.pagelayer-animation-pop:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-pop{
animation-name:pagelayer-animation-pop;
animation-duration:0.4s;
animation-timing-function:linear;
animation-iteration-count:1
}

@keyframes pagelayer-animation-buzz{
50%{
transform:translateX(3px) rotate(2deg)
}
100%{
transform:translateX(-3px) rotate(-2deg)
}
}

.pagelayer-animation-buzz:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-buzz{
animation-name:pagelayer-animation-buzz;
animation-duration:0.15s;
animation-timing-function:linear;
animation-iteration-count:infinite;
}

@keyframes pagelayer-animation-buzz-out{
10%{transform:translateX(3px) rotate(2deg)}
20%{transform:translateX(-3px) rotate(-2deg)}
30%{transform:translateX(3px) rotate(2deg)}
40%{transform:translateX(-3px) rotate(-2deg)}
50%{transform:translateX(2px) rotate(1deg)}
60%{transform:translateX(-2px) rotate(-1deg)}
70%{transform:translateX(2px) rotate(1deg)}
80%{transform:translateX(-2px) rotate(-1deg)}
90%{transform:translateX(1px) rotate(0)}
100%{transform:translateX(-1px) rotate(0)}
}

.pagelayer-animation-buzz-out:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-buzz-out{
animation-name:pagelayer-animation-buzz-out;
animation-duration:0.7s;
animation-timing-function:linear;
animation-iteration-count:1;
}

.pagelayer-animation-float:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-float{
transform:translateY(-8px)
}

.pagelayer-animation-sink:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-sink{
transform:translateY(8px)
}

@keyframes pagelayer-animation-bob{
0%{transform:translateY(-8px)}
50%{transform:translateY(-4px)}
100%{transform:translateY(-8px)}
}

@keyframes pagelayer-animation-bob-up{
100%{transform:translateY(-8px)}
}

.pagelayer-animation-bob:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-bob{
animation-name:pagelayer-animation-bob-up,pagelayer-animation-bob;
animation-duration:0.4s,1.5s;
animation-timing-function:ease-out,ease-in-out;
animation-delay:0s,0.3s;
animation-iteration-count:infinite;
}

@keyframes pagelayer-animation-hang{
0%{transform:translateY(8px)}
50%{transform:translateY(4px)}
100%{transform:translateY(8px)}
}

@keyframes pagelayer-animation-hang-up{
100%{transform:translateY(8px)}
}

.pagelayer-animation-hang:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-hang{
animation-name:pagelayer-animation-hang-up,pagelayer-animation-hang;
animation-duration:0.4s,1.5s;
animation-timing-function:ease-out,ease-in-out;
animation-delay:0s,0.3s;
animation-iteration-count:1,infinite;
animation-direction:normal,alternate;
}

.pagelayer-animation-bounce-in{
transition-duration:0.5s;
}
.pagelayer-animation-bounce-in:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-bounce-in{
transform:scale(1.2);
transition-timing-function:cubic-bezier(0.52,2.07,0.36,-0.41);
}

.pagelayer-animation-bounce-out{
transition-duration:0.5s;
}

.pagelayer-animation-bounce-out:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-bounce-out{
transform:scale(0.8);
transition-timing-function:cubic-bezier(0.52,2.07,0.36,-0.41);
}

.pagelayer-animation-rotate:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-rotate{
transform:rotate(6deg);
}

.pagelayer-animation-grow-rotate:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-grow-rotate{
transform:scale(1.1) rotate(6deg);
}

.pagelayer-animation-skew-forward:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-skew-forward{
transform:skew(-9deg);
}

.pagelayer-animation-skew-backward:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-skew-backward{
transform:skew(9deg);
}

@keyframes pagelayer-animation-wobble-vertical{
17%{transform:translateY(9px)}
33%{transform:translateY(-7px)}
45%{transform:translateY(5px)}
67%{transform:translateY(-3px)}
83%{transform:translateY(1px)}
100%{transform:translateY(0)}
}

.pagelayer-animation-wobble-vertical:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-wobble-vertical{
animation-name:pagelayer-animation-wobble-vertical;
animation-duration:1s;
animation-timing-function:ease-in-out;
animation-iteration-count:1;
}

@keyframes pagelayer-animation-wobble-horizontal{
17%{transform:translateX(9px)}
33%{transform:translateX(-7px)}
50%{transform:translatex(5px)}
67%{transform:translateX(-3px)}
83%{transform:translateX(1px)}
100%{transform:translateX(0)}
}

.pagelayer-animation-wobble-horizontal:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-wobble-horizontal{
animation-name:pagelayer-animation-wobble-horizontal;
animation-duration:1s;
animation-timing-function:ease-in-out;
animation-iteration-count:1;
}

@keyframes pagelayer-animation-wobble-bottom-to-right{
17%{transform:translate(9px,9px)}
33%{transform:translate(-7px,-7px)}
50%{transform:translate(5px,5px)}
67%{transform:translate(-3px,-3px)}
83%{transform:translate(1px,1px)}
100%{transform:translate(0,0)}
}


.pagelayer-animation-wobble-bottom-to-right:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-wobble-bottom-to-right{
animation-name:pagelayer-animation-wobble-bottom-to-right;
animation-duration:1s;
animation-timing-function:ease-in-out;
animation-iteration-count:1;
}

@keyframes pagelayer-animation-wobble-top-to-right{
17%{transform:translate(9px,-9px)}
33%{transform:translate(-7px,7px)}
50%{transform:translate(5px,-5px)}
67%{transform:translate(-3px,3px)}
83%{transform:translate(1px,-1px)}
100%{transform:translate(0,0)}
}


.pagelayer-animation-wobble-top-to-right:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-wobble-top-to-right{
animation-name:pagelayer-animation-wobble-top-to-right;
animation-duration:1s;
animation-timing-function:ease-in-out;
animation-iteration-count:1;
}

.pagelayer-animation-wobble-top{
transform-origin:0 100%;
}

@keyframes pagelayer-animation-wobble-top{
17%{transform:skew(-13deg)}
33%{transform:skew(11deg)}
50%{transform:skew(-7deg)}
67%{transform:skew(5deg)}
83%{transform:skew(-3deg)}
100%{transform:skew(0)}
}

.pagelayer-animation-wobble-top:hover,
.pagelayer-anim-par:hover .pagelayer-animation-wobble-top{
animation-name:pagelayer-animation-wobble-top;
animation-duration:1s;
animation-timing-function:ease-in-out;
animation-iteration-count:1
}

.pagelayer-animation-wobble-bottom{
transform-origin:100% 0;
}

@keyframes pagelayer-animation-wobble-bottom{
17%{transform:skew(-13deg)}
33%{transform:skew(11deg)}
50%{transform:skew(-7deg)}
67%{transform:skew(5deg)}
83%{transform:skew(-3deg)}
100%{transform:skew(0)}
}

.pagelayer-animation-wobble-bottom:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-wobble-bottom{
animation-name:pagelayer-animation-wobble-bottom;
animation-duration:1s;
animation-timing-function:ease-in-out;
animation-iteration-count:1
}

@keyframes pagelayer-animation-wobble-skew{
17%{transform:skew(-13deg)}
33%{transform:skew(11deg)}
50%{transform:skew(-7deg)}
67%{transform:skew(5deg)}
83%{transform:skew(-3deg)}
100%{transform:skew(0)}
}

.pagelayer-animation-wobble-skew:hover, 
.pagelayer-anim-par:hover .pagelayer-animation-wobble-skew{
animation-name:pagelayer-animation-wobble-skew;
animation-duration:1s;
animation-timing-function:ease-in-out;
animation-iteration-count:1
}

/* Icon animation end */

/*Badge widget */
.pagelayer-badge a{
text-decoration: none !important;
}
.pagelayer-badge-btn{
display:none;
padding:10px;
padding: 0.40em 0.36em;
}
 
.pagelayer-badge-details{
margin-left:1px;
border-radius:.25rem;
padding: 0.10rem;
position: relative;
top: -3px;
}
 
.pagelayer-badge-details.pagelayer-badge-normal{
border-radius:.25rem;
}

.pagelayer-badge-details.pagelayer-badge-pills{
border-radius:10rem;
}

.pagelayer-badge-btn .pagelayer-badge-details{
position:relative;
top:-1px;
} 

/*Badge widget end*/

/*Tooltip widget*/
.pagelayer-tooltip-container{
position:relative;
display:inline-block;
line-height: 1;
}

.pagelayer-tooltip-text{
visibility: hidden;
width: 300px;
background-color: #000000;
color: #ffffff;
text-align: center;
border-radius: 6px;
position: absolute;
z-index: 1;
padding:5px;
word-break:break-word;
}

.pagelayer-tooltip-icon{
margin:0px 4px;
}
.pagelayer-tooltip-top{
bottom: calc(100% + 10px);
left: 50%;
transform: translateX(-50%);
}

.pagelayer-tooltip-top:after{
content: "";
position: absolute;
top: 100%;
left: 50%;
margin-left: -5px;
border-width: 5px;
border-style: solid;
border-color: #000000 transparent transparent transparent;
}

.pagelayer-tooltip-right{
top: 50%;
left: calc(100% + 10px);
transform: translateY(-50%);
}

.pagelayer-tooltip-right:after {
content: "";
position: absolute;
top: 50%;
right: 100%;
margin-top: -5px;
border-width: 5px;
border-style: solid;
border-color: transparent #000000 transparent transparent;
}

.pagelayer-tooltip-bottom{
top: calc(100% + 10px);
left: 50%;
transform: translateX(-50%);
}

.pagelayer-tooltip-bottom:after{
content: "";
position: absolute;
bottom: 100%;
left: 50%;
margin-left: -5px;
border-width: 5px;
border-style: solid;
border-color: transparent transparent #000000 transparent;
}

.pagelayer-tooltip-left{
top: 50%;
bottom: auto;
right: calc(100% + 10px);
transform: translateY(-50%);
}

.pagelayer-tooltip-left:after{
content: "";
position: absolute;
top: 50%;
left: 100%;
transform: translateY(-50%);
margin-top: -5px;
border-width: 5px;
border-style: solid;
border-color: transparent transparent transparent #000000;
}

.pagelayer-tooltip-container:hover .pagelayer-tooltip-text,
.pagelayer-tooltip-text:focus{
visibility: visible !important;
}

.pagelayer-tooltip-on-click{
visibility: visible !important;
}
/*Tooltip widget end*/

/* Button widget */

.pagelayer-btn-holder{
border-radius:5px;
display:inline-block;
line-height:1em;
transition:all 0.3s;
cursor: pointer;
box-sizing:border-box;
}

.pagelayer-btn-default{
background-color:#818a91 !important;
color:#ffffff !important;
}

.pagelayer-btn-primary,
.pagelayer-badge-primary,
.pagelayer-progress-primary{
color:#ffffff !important;
background-color:#007bff !important;
}

.pagelayer-btn-primary:hover{
color:#ffffff !important;
background-color: #0069d9 !important;
}

.pagelayer-btn-secondary,
.pagelayer-badge-secondary,
.pagelayer-progress-secondary{
color:#ffffff !important;
background-color:#6c757d !important;
}

.pagelayer-btn-secondary:hover{
color: #ffffff !important;
background-color: #5a6268 !important;
}

.pagelayer-btn-success,
.pagelayer-badge-success,
.pagelayer-progress-success{
color: #fff;
background-color: #28a745 !important;
}

.pagelayer-btn-success:hover{
color: #ffffff !important;
background-color: #218838 !important;
}

.pagelayer-btn-info,
.pagelayer-badge-info,
.pagelayer-progress-info{
color: #ffffff !important;
background-color: #17a2b8 !important;
}

.pagelayer-btn-info:hover{
color: #ffffff !important;
background-color: #138496 !important;
}

.pagelayer-btn-warning,
.pagelayer-badge-warning,
.pagelayer-progress-warning{
color: #212529 !important;
background-color: #ffc107 !important;
}

.pagelayer-btn-warning:hover{
color: #212529 !important;
background-color: #e0a800 !important;
}

.pagelayer-btn-danger,
.pagelayer-badge-danger,
.pagelayer-progress-danger{
color: #ffffff !important;
background-color: #dc3545 !important;
}

.pagelayer-btn-danger:hover{
color: #ffffff !important;
background-color: #c82333 !important;
}

.pagelayer-btn-dark,
.pagelayer-badge-dark,
.pagelayer-progress-dark{
color: #ffffff !important;
background-color: #343a40 !important;
}

.pagelayer-btn-dark:hover{
color: #ffffff !important;
background-color: #23272b !important;
}

.pagelayer-btn-light,
.pagelayer-badge-light,
.pagelayer-progress-light{
color: #212529 !important;
background-color: #f8f9fa !important;
}

.pagelayer-btn-light:hover{
color: #212529 !important;
background-color: #e2e6ea !important;
}

.pagelayer-btn-link{
color: #007bff !important;
}

.pagelayer-btn-link:hover{
text-decoration: underline !important;
}

.pagelayer-btn-default:hover{
color:#ffffff;
}

.pagelayer-btn-mini{
font-size: 14px;
padding: 10px 20px;
}

.pagelayer-btn-small{
font-size: 16px;
padding: 15px 30px;
}

.pagelayer-btn-large{
font-size: 18px;
padding: 20px 40px;
}

.pagelayer-btn-extra-large{
font-size: 20px;
padding: 25px 50px;
}

.pagelayer-btn-double-large{
font-size: 22px;
padding: 30px 60px;
}

.pagelayer-btn-icon-left .pagelayer-btn-icon:last-child,
.pagelayer-btn-icon-right .pagelayer-btn-icon:first-child,
.pagelayer-btn-icon-left .pagelayer-btn-load-icon:last-child,
.pagelayer-btn-icon-right .pagelayer-btn-load-icon:first-child,
.pagelayer-btn-icon-left .pagelayer-cf-icon-right,
.pagelayer-btn-icon-right .pagelayer-cf-icon-left{
display: none;
}

.pagelayer-btn-icon-left .pagelayer-btn-icon,
.pagelayer-btn-icon-left .pagelayer-btn-load-icon{
padding-left:0 !important;
position: relative;
z-index: 1;
}

.pagelayer-btn-icon-right .pagelayer-btn-icon,
.pagelayer-btn-icon-right .pagelayer-btn-load-icon{
padding-right:0 !important;
position: relative;
z-index: 1;
}

/* Button widget end */

/* Social Profile */
.pagelayer-icon-holder[class*="pagelayer-facebook"] .pagelayer-social-fa{
color:#3B5998;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-facebook"],
.pagelayer-share-content[class*="pagelayer-facebook"]{
background-color:#3B5998;
color:#3B5998;
}

.pagelayer-icon-holder[class*="pagelayer-twitter"] .pagelayer-social-fa{
color:#1DA1F2;	
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-twitter"],
.pagelayer-share-content[class*="pagelayer-twitter"]{
background-color:#1DA1F2;
color:#1DA1F2;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-x-twitter"],
.pagelayer-share-content[class*="pagelayer-x-twitter"]{
background-color:#000000;
}

.pagelayer-icon-holder[class*="pagelayer-android"] .pagelayer-social-fa{
color:#A4C639
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-android"],
.pagelayer-share_grp[class*="pagelayer-social-shape"] .pagelayer-share-content[class*="pagelayer-android"]{
background-color:#A4C639;
}

.pagelayer-icon-holder[class*="pagelayer-google-plus"] .pagelayer-social-fa{
color:#DD4B39
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-google-plus"],
.pagelayer-share-content[class*="pagelayer-google-plus"]{
background-color:#DD4B39;
color:#DD4B39;
}

.pagelayer-icon-holder[class*="pagelayer-instagram"] .pagelayer-social-fa{
color:#262626;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-instagram"],
.pagelayer-share-content[class*="pagelayer-instagram"]{
background-color:#1DA1F2;
color:#262626;
}

.pagelayer-icon-holder[class*="pagelayer-linkedin"] .pagelayer-social-fa{
color:#0077B5;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-linkedin"],
.pagelayer-share-content[class*="pagelayer-linkedin"]{
background-color:#0077B5;
color:#0077B5;
}

.pagelayer-icon-holder[class*="pagelayer-behance"] .pagelayer-social-fa{
color:#053eff;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-behance"],
.pagelayer-share-content[class*="pagelayer-behance"]{
background-color:#053eff;
color:#053eff;
}

.pagelayer-icon-holder[class*="pagelayer-pinterest"] .pagelayer-social-fa{
color:#c8232c;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-pinterest"],
.pagelayer-share-content[class*="pagelayer-pinterest"]{
background-color:#c8232c;
color:#c8232c;
}

.pagelayer-icon-holder[class*="pagelayer-get-pocket"] .pagelayer-social-fa{
color:#ef4156;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-get-pocket"],
.pagelayer-share-content[class*="pagelayer-get-pocket"]{
background-color:#ef4156;
color:#ef4156;
}

.pagelayer-icon-holder[class*="pagelayer-envelope"] .pagelayer-social-fa{
color:#292929;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-envelope"],
.pagelayer-share-content[class*="pagelayer-envelope"]{
background-color:#292929;
color:#292929;
}

.pagelayer-icon-holder[class*="pagelayer-reddit"] .pagelayer-social-fa{
color:#ff4301;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-reddit"],
.pagelayer-share-content[class*="pagelayer-reddit"]{
background-color:#ff4301;
color:#ff4301;
}

.pagelayer-icon-holder[class*="pagelayer-rss"] .pagelayer-social-fa{
color:#F26522;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-rss"],
.pagelayer-share-content[class*="pagelayer-rss"]{
background-color:#F26522;
color:#F26522;
}

.pagelayer-icon-holder[class*="pagelayer-skype"] .pagelayer-social-fa{
color:#00aff0;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-skype"],
.pagelayer-share-content[class*="pagelayer-skype"]{
background-color:#00aff0;	
color:#00aff0;
}

.pagelayer-icon-holder[class*="pagelayer-slideshare"] .pagelayer-social-fa{
color:#0077b5;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-slideshare"],
.pagelayer-share-content[class*="pagelayer-slideshare"]{
background-color:#0077b5;
color:#0077b5;
}

.pagelayer-icon-holder[class*="pagelayer-snapchat"] .pagelayer-social-fa{
color:#fffc00;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-snapchat"],
.pagelayer-share-content[class*="pagelayer-snapchat"]{
background-color:#fffc00;
color:#fffc00;
}

.pagelayer-icon-holder[class*="pagelayer-soundcloud"] .pagelayer-social-fa{
color:#ff8800;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-soundcloud"],
.pagelayer-share-content[class*="pagelayer-soundcloud"]{
background-color:#ff8800;
color:#ff8800;
}

.pagelayer-icon-holder[class*="pagelayer-spotify"] .pagelayer-social-fa{
color:#1ED760;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-spotify"],
.pagelayer-share-content[class*="pagelayer-spotify"]{
background-color:#1ED760;
color:#1ED760;
}

.pagelayer-icon-holder[class*="pagelayer-stack-overflow"] .pagelayer-social-fa{
color:#F48024;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-stack-overflow"],
.pagelayer-share-content[class*="pagelayer-stack-overflow"]{
background-color:#F48024;
color:#F48024;
}

.pagelayer-icon-holder[class*="pagelayer-steam"] .pagelayer-social-fa{
color:#00adee;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-steam"],
.pagelayer-share-content[class*="pagelayer-steam"]{
background-color:#00adee;
color:#00adee;
}

.pagelayer-icon-holder[class*="pagelayer-stumbleupon"] .pagelayer-social-fa{
color:#EB4924;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-stumbleupon"],
.pagelayer-share-content[class*="pagelayer-stumbleupon"]{
background-color:#EB4924;
color:#EB4924;
}

.pagelayer-icon-holder[class*="pagelayer-telegram"] .pagelayer-social-fa{
color:#0088cc;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-telegram"],
.pagelayer-share-content[class*="pagelayer-telegram"]{
background-color:#0088cc;
color:#0088cc;
}

.pagelayer-icon-holder[class*="pagelayer-thumb-tack"] .pagelayer-social-fa{
color:#1AA1D8;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-thumb-tack"],
.pagelayer-share-content[class*="pagelayer-thumb-tack"]{
background-color:#1AA1D8;
color:#1AA1D8;
}

.pagelayer-icon-holder[class*="pagelayer-tripadvisor"] .pagelayer-social-fa{
color:#00af87;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-tripadvisor"],
.pagelayer-share-content[class*="pagelayer-tripadvisor"]{
background-color:#00af87;
color:#00af87;
}

.pagelayer-icon-holder[class*="pagelayer-tumblr"] .pagelayer-social-fa{
color:#001935;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-tumblr"],
.pagelayer-share-content[class*="pagelayer-tumblr"]{
background-color:#001935;
color:#001935;
}

.pagelayer-icon-holder[class*="pagelayer-twitch"] .pagelayer-social-fa{
color:#6441a5;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-twitch"],
.pagelayer-share-content[class*="pagelayer-twitch"]{
background-color:#6441a5;
color:#6441a5;
}

.pagelayer-icon-holder[class*="pagelayer-vimeo"] .pagelayer-social-fa{
color:#1CB7EA;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-vimeo"],
.pagelayer-share-content[class*="pagelayer-vimeo"]{
background-color:#1CB7EA;
color:#1CB7EA;
}

.pagelayer-icon-holder[class*="pagelayer-vk"] .pagelayer-social-fa{
color:#4C75A3;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-vk"],
.pagelayer-share-content[class*="pagelayer-vk"]{
background-color:#4C75A3;
color:#4C75A3;
}

.pagelayer-icon-holder[class*="pagelayer-weibo"] .pagelayer-social-fa{
color:#DF2029;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-weibo"],
.pagelayer-share-content[class*="pagelayer-weibo"]{
background-color:#DF2029;
color:#DF2029;
}

.pagelayer-icon-holder[class*="pagelayer-weixin"] .pagelayer-social-fa{
color:#7bb32e;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-weixin"],
.pagelayer-share-content[class*="pagelayer-weixin"]{
background-color:#7bb32e;
color:#7bb32e;
}

.pagelayer-icon-holder[class*="pagelayer-whatsapp"] .pagelayer-social-fa{
color:#25D366;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-whatsapp"],
.pagelayer-share-content[class*="pagelayer-whatsapp"]{
background-color:#25D366;
color:#25D366;
}

.pagelayer-icon-holder[class*="pagelayer-wordpress"] .pagelayer-social-fa{
color:#21759b;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-wordpress"],
.pagelayer-share-content[class*="pagelayer-wordpress"]{
background-color:#21759b;
color:#21759b;
}

.pagelayer-icon-holder[class*="pagelayer-xing"] .pagelayer-social-fa{
color:#026466;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-xing"],
.pagelayer-share-content[class*="pagelayer-xing"]{
background-color:#026466;	
color:#026466;
}

.pagelayer-icon-holder[class*="pagelayer-yelp"] .pagelayer-social-fa{
color:#af0606;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-yelp"],
.pagelayer-share-content[class*="pagelayer-yelp"]{
background-color:#af0606;
color:#af0606;
}

.pagelayer-icon-holder[class*="pagelayer-youtube"] .pagelayer-social-fa{
color:#ff0000;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-youtube"],
.pagelayer-share-content[class*="pagelayer-youtube"]{
background-color:#ff0000;
color:#ff0000;
}

.pagelayer-icon-holder[class*="pagelayer-500px"] .pagelayer-social-fa{
color:#0099e5;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-500px"],
.pagelayer-share-content[class*="pagelayer-500px"]{
background-color:#0099e5;
color:#0099e5;
}

.pagelayer-icon-holder[class*="pagelayer-flickr"] .pagelayer-social-fa{
color:#0063dc;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-flickr"],
.pagelayer-share-content[class*="pagelayer-flickr"]{
background-color:#0063dc;
color:#0063dc;
}

.pagelayer-icon-holder[class*="pagelayer-github"] .pagelayer-social-fa{
color:#4078c0;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-github"],
.pagelayer-share-content[class*="pagelayer-github"]{
background-color:#4078c0;	
color:#4078c0;
}

.pagelayer-icon-holder[class*="pagelayer-gitlab"] .pagelayer-social-fa{
color:#fca326;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-gitlab"],
.pagelayer-share-content[class*="pagelayer-gitlab"]{
background-color:#fca326;
color:#fca326;
}

.pagelayer-icon-holder[class*="pagelayer-apple"] .pagelayer-social-fa{
color:#999999;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-apple"],
.pagelayer-share-content[class*="pagelayer-apple"]{
background-color:#999999;
color:#999999;
}

.pagelayer-icon-holder[class*="pagelayer-jsfiddle"] .pagelayer-social-fa{
color:#0084FF;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-jsfiddle"],
.pagelayer-share-content[class*="pagelayer-jsfiddle"]{
background-color:#0084FF;
color:#0084FF;
}

.pagelayer-icon-holder[class*="pagelayer-houzz"] .pagelayer-social-fa{
color:#7ac142;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-houzz"],
.pagelayer-share-content[class*="pagelayer-houzz"]{
background-color:#7ac142;
color:#7ac142;
}

.pagelayer-icon-holder[class*="pagelayer-bitbucket"] .pagelayer-social-fa{
color:#205081;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-bitbucket"],
.pagelayer-share-content[class*="pagelayer-bitbucket"]{
background-color:#205081;
color:#205081;
}

.pagelayer-icon-holder[class*="pagelayer-codepen"] .pagelayer-social-fa{
color:#0ebeff;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-codepen"],
.pagelayer-share-content[class*="pagelayer-codepen"]{
background-color:#0ebeff;
color:#0ebeff;
}

.pagelayer-icon-holder[class*="pagelayer-delicious"] .pagelayer-social-fa{
color:#3399ff;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-delicious"],
.pagelayer-share-content[class*="pagelayer-delicious"]{
background-color:#3399ff;
color:#3399ff;
}

.pagelayer-icon-holder[class*="pagelayer-medium"] .pagelayer-social-fa{
color:#00ab6c;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-medium"],
.pagelayer-share-content[class*="pagelayer-medium"]{
background-color:#00ab6c;	
color:#00ab6c;
}

.pagelayer-icon-holder[class*="pagelayer-meetup"] .pagelayer-social-fa{
color:#e0393e;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-meetup"],
.pagelayer-share-content[class*="pagelayer-meetup"]{
background-color:#e0393e;	
color:#e0393e;
}

.pagelayer-icon-holder[class*="pagelayer-mixcloud"] .pagelayer-social-fa{
color:#52aad8;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-mixcloud"],
.pagelayer-share-content[class*="pagelayer-mixcloud"]{
background-color:#52aad8;	
color:#52aad8;
}

.pagelayer-icon-holder[class*="pagelayer-dribbble"] .pagelayer-social-fa{
color:#444444;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-dribbble"],
.pagelayer-share-content[class*="pagelayer-dribbble"]{
background-color:#444444;
color:#444444;
}

.pagelayer-icon-holder[class*="pagelayer-foursquare"] .pagelayer-social-fa{
color:#f94877;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-icon-holder[class*="pagelayer-foursquare"],
.pagelayer-share-content[class*="pagelayer-foursquare"]{
background-color:#f94877;
color:#f94877;
}

.pagelayer-share_grp .pagelayer-social-fa,
.pagelayer-share_grp .pagelayer-social-fa:before{
position: absolute;
top: 50%;
left: 50%;
transform: translate(-50% , -50%);
}
.pagelayer-share_grp{
font-size: 0;
}
.pagelayer-share_grp > div{
display: inline-block;
}

.pagelayer-share_grp .pagelayer-social-fa{
height:1em;
width:1em;
}

.pagelayer-share_grp .pagelayer-icon-holder{
position: relative;
min-height: 1em;
min-width: 1em;
}

.pagelayer-share-content .pagelayer-social-fa,
.pagelayer-share-content span{
color: inherit;
}

.pagelayer-share_grp.pagelayer-social-bg-none .pagelayer-share-content,
.pagelayer-share_grp.pagelayer-social-outline-border .pagelayer-share-content{
background-color:unset;
}

.pagelayer-share_grp.pagelayer-social-outline-border .pagelayer-share-content{
border:2px solid;
}

.pagelayer-social_grp[class*="pagelayer-social-shape"] .pagelayer-social-fa,
.pagelayer-share_grp[class*="pagelayer-social-shape"] .pagelayer-social-fa,
.pagelayer-share_grp[class*="pagelayer-social-shape"] .pagelayer-icon-name span{
color: #ffffff;
}

.pagelayer-social_grp .pagelayer-icon-holder{
display:inline-block;
line-height: 100%;
text-align: center;
}

.pagelayer-share-content,
.pagelayer-share-content .pagelayer-icon-name{
display:flex;
}

.pagelayer-share-content .pagelayer-icon-name{
align-items:center;
}

.pagelayer-social-shape-circle .pagelayer-share-content{
border-radius:100000px;
}

.pagelayer-icon-name span{
padding:0 20px 0 5px;
}

.pagelayer-share-buttons{
position:relative;
border-radius: inherit;
}

.pagelayer-social-shape-boxed .pagelayer-share-buttons:before{
content: "";
position: absolute;
width: 100%;
height: 100%;
background: rgba(0,0,0,0.15);
border-top-left-radius: inherit;
border-bottom-left-radius: inherit;
left: 0;
}

.pagelayer-share_grp .pagelayer-icon-name span,
.pagelayer-share_grp .pagelayer-icon-holder{
display:none;
}

.pagelayer-share-type-icon .pagelayer-icon-holder,
.pagelayer-share-type-icon-label .pagelayer-icon-holder,
.pagelayer-share-type-label .pagelayer-icon-name span,
.pagelayer-share-type-icon-label .pagelayer-icon-name span{
display:block;
}

/* Social Profile end */

.pagelayer-list-icon-holder{
display: -webkit-box;
display: -webkit-flex;
display: -ms-flexbox;
display: flex;
-webkit-box-align: start;
-webkit-align-items: flex-start;
-ms-flex-align: start;
align-items: flex-start;
}

.pagelayer-list-icon,
.pagelayer-list-item{
-webkit-align-self: center;
-ms-flex-item-align: center;
align-self: center;
}

.pagelayer-list-item{
width:100%;
}

.pagelayer-list-ul{
margin:0;
padding:0;	
}

/* Video */

.pagelayer-video .pagelayer-video-holder{
position:relative;
width: 100%;
height: auto;
overflow: hidden;
}

.pagelayer-video-iframe{
position:absolute;
width:100%;
height:100%;
top:0;
left:0;
}

.pagelayer-video-aspect-1-1{
padding-top:100%;
}

.pagelayer-video-aspect-3-2{
padding-top:66.66%;
}

.pagelayer-video-aspect-4-3{
padding-top: 75%;
}

.pagelayer-video-aspect-8-5{
padding-top:62.5%;
}

.pagelayer-video-aspect-16-9{
padding-top: 56.25%;
}

.pagelayer-video .pagelayer-video-holder .pagelayer-video-overlay{
position:absolute;
top:0;
left:0;
right:0;
bottom:0;
background-size: cover;
background-position: center center;
background-repeat: no-repeat;
cursor: pointer;
}

.pagelayer-video .pagelayer-video-holder a{
position:absolute;
top:0;
left:0;
right:0;
bottom:0;
}

.pagelayer-video .pagelayer-video-holder .pagelayer-video-overlay i{
position: absolute;
top: 50%;
left: 50%;
-webkit-transform: translateX(-50%) translateY(-50%);
-ms-transform: translateX(-50%) translateY(-50%);
transform: translateX(-50%) translateY(-50%);
}
/* Video end */

/** Image **/

.pagelayer-image, .pagelayer-image .pagelayer-image-holder{
position: relative;
}

.pagelayer-image-link{
text-decoration:none;
cursor: pointer;
}

.pagelayer-image-caption{
margin-bottom: unset;
}

.pagelayer-image-overlay{
position: absolute;
width: 100%;
top: 0;
height: 100%;
left: 0;
opacity:0;
overflow:auto;
transition: .5s ease;
}

.pagelayer-image:hover .pagelayer-image-overlay{
opacity:1;
}

.pagelayer-image .pagelayer-image-overlay .pagelayer-image-overlay-content{
width:100%;
}

.pagelayer-image *{
border-radius:inherit;
}

/** Image End **/

.pagelayer-button {
text-align: center
}

.pagelayer-button i{
padding: 0 3px;
}

.pagelayer-audio-container, .pagelayer-audio-container audio{
width:100%;
}

.pagelayer-testimonial-avatar img{
border: 1px solid #eee;
border-radius: 50%;
margin-right: 10px;
}

.pagelayer-alignment-center{
text-align: center;
}

.pagelayer-alignment-left{
text-align: left;
}

.pagelayer-alignment-right{
text-align: right;
}

/* Testimonial */
.pagelayer-testimonial{
padding:0.4em 0.8em;
line-height: normal;
}

.pagelayer-testimonial-image{
-o-object-fit: cover;
object-fit: cover;
display: unset !important;
}

.pagelayer-testimonial-square{
border-radius:0px;
}

.pagelayer-testimonial-circle{
border-radius:50%;
}

.pagelayer-aside-position{
display:inline-block;
vertical-align: middle;
}

.pagelayer-aside-position .pagelayer-testimonial-cite{
text-align: left;
}
.pagelayer-top-position, .pagelayer-testimonial-author{
display:block;
}

.pagelayer-aside-position .pagelayer-testimonial-author{
position: relative;
}

.pagelayer-testimonial-container .pagelayer-testimonial-avatar,
.pagelayer-testimonial-container .pagelayer-testimonial-details{
display: table-cell;
vertical-align: middle;
}

.pagelayer-testimonial-designation{
color: #999;
font-size: 12px;
}

/* Testimonial End */

/* Progress bar */
.pagelayer-progress-goal{
margin-top: 10px;
}

.pagelayer-progress-container{
background-color:#eeeeee;
}

.pagelayer-progress-bar{
overflow:hidden;
}

.pagelayer-progress-percent{
float:right;
padding-right:10px;
}

.pagelayer-progress-text{
float:left;
padding-left:10px;
}

.pagelayer-progress-prefix {
float: left;
text-align: left;
display: inline-block;
}

.pagelayer-progress-suffix{
text-align: right;
}

/*Progress bar end*/

/*Stars widget start*/

.pagelayer-stars > *{
vertical-align:middle;
}
.pagelayer-stars-container{
text-align:center;
color: #ccd6df;
font-family: "FontAwesome", "Font Awesome 5 Free";
display: inline-block;
position: relative;
border-color: 1px solid #ccd6df;
}

.pagelayer-stars-container .pagelayer-stars-icon{
position: relative;
display:inline-block;
line-height: 1;
}

.pagelayer-stars-container .pagelayer-stars-icon:before{
content: "\f005";
font-size: inherit;
font-family: inherit;
overflow: hidden;
color: #f0ad4e;
position:absolute;
font-weight:900;
top:0;
left:0;
}

.pagelayer-stars-icon.pagelayer-stars-empty:before{
width:0em;
}

.pagelayer-stars-icon.pagelayer-stars-1:before{
width:0.1em;
}

.pagelayer-stars-icon.pagelayer-stars-2:before{
width:0.2em;
}

.pagelayer-stars-icon.pagelayer-stars-3:before{
width:0.3em;
}

.pagelayer-stars-icon.pagelayer-stars-4:before{
width:0.4em;
}

.pagelayer-stars-icon.pagelayer-stars-5:before{
width:0.5em;
}

.pagelayer-stars-icon.pagelayer-stars-6:before{
width:0.6em;
}

.pagelayer-stars-icon.pagelayer-stars-7:before{
width:0.7em;
}

.pagelayer-stars-icon.pagelayer-stars-8:before{
width:0.8em;
}

.pagelayer-stars-icon.pagelayer-stars-9:before{
width:0.9em;
}

.pagelayer-stars-icon.pagelayer-stars-full:before{
width:1.1em;
}
/*Stars widget end*/

/* Site Title Start */
.pagelayer-wp-title-align-top{
display:block;
text-align: center;
}

.pagelayer-wp-title-heading{
padding:5px 20px;
font-size: 20px;
font-weight: 500;
text-decoration-style: solid !important;
margin: 0px;
transition: all 400ms;
flex-grow:1;
}

.pagelayer-wp-title-img{
box-shadow: none !important;
}

.pagelayer-wp-title-desc{
padding: 5px 20px;
}

.pagelayer-wp-title-align-left,
.pagelayer-wp-title-align-right{
display: flex;
}

.pagelayer-wp-title-align-right{
text-align: right;
-webkit-box-orient: horizontal;
-webkit-box-direction: reverse;
flex-direction: row-reverse;
}

.pagelayer-wp-title-vertical-top{
align-items: flex-start;
-webkit-align-items: flex-start;
-webkit-box-align: start;
-ms-flex-align: start;
}

.pagelayer-wp-title-vertical-middle{
align-items: center;
-webkit-align-items: center;
-webkit-box-align: center;
-ms-flex-align: center;
}

.pagelayer-wp-title-vertical-bottom{
align-items: flex-end;
-webkit-align-items: flex-end;
-webkit-box-align: end;
-ms-flex-align: end;
}

.pagelayer-wp-title-section .pagelayer-wp-title-link{
text-decoration: none !important;
}
/* Site title End */

/*pricing plans*/
.pagelayer-pricing{
text-align:center;
border:2px solid #e8e3e3;
border-radius:6px;
}

.pagelayer-pricing-rate-section{
padding:20px;
}

.pagelayer-pricing-details h1,
.pagelayer-pricing-details h2,
.pagelayer-pricing-details h3,
.pagelayer-pricing-details h4,
.pagelayer-pricing-details h5{
margin:10px 0px; padding:0px;
}

.pagelayer-pricing-details{
position: relative;
background-color:#1e1558;
border-top-left-radius: inherit;
border-top-right-radius: inherit;
}

.pagelayer-pricing-details .pagelayer-pricing-type{
text-transform: uppercase;
}

.pagelayer-pricing-sub-title{
font-weight: normal;
}

.pagelayer-pricing-details .pagelayer-pricing-price{
font-weight:800;
}

.pagelayer-pricing-price.pagelayer-pricing-original{
display:none;
text-decoration: line-through;
margin-right: 10px;
}
  
h2.pagelayer-pricing-price{
display:inline-block;
margin:5px 0px;
}

.pagelayer-pricing-details{
font-size:15px;  
}

.pagelayer-pricing-duration{
font-size:15px;
margin:0px;
}

.pagelayer-pricing-features{
padding:20px;
border-bottom-left-radius: inherit;
border-bottom-right-radius: inherit;
}

.pagelayer-pricing-features .pagelayer-pricing-ul{
padding:0px;
}

.pagelayer-pricing-ul li{
display: inline-block;
list-style-type: none;
padding:5px 0px;
}

.pagelayer-pricing-ul .pagelayer-list_item:after{
border-bottom:2px solid #c5c5c5;
margin-top: 5px;
}

.pagelayer-list-li span i{
margin-right:5px;
}

.pagelayer-pricing-btn{
display: inline-block;
}

.pagelayer-pricing-additional{
margin:20px 0px 0px;
}

.pagelayer-pricing-ribbon-container{
position: absolute;
top: 0;
left: auto;
right: 0;
transform: rotate(90deg);
width: 150px;
overflow: hidden;
height: 150px;
}

.pagelayer-pricing-ribbon{
display: none;
width: 200%;
background-color: #ce4210ff;
position: absolute;
left:0;
text-align: center;
line-height: 2;
letter-spacing: 1px;
color: #f0f0f0;
margin-top: 40px;
transform: translateY(-50%) translateX(-50%) translateX(50px) rotate(-45deg);
}

.pagelayer-pricing-currency-top{
vertical-align: top;
}

.pagelayer-pricing-currency-middle{
vertical-align: middle;
}

.pagelayer-pricing-currency-bottom{
vertical-align: bottom;
}

/*pricing plans end*/

/* Quote */

.pagelayer-quotation-overlay{
position: absolute;
font-size: 70px;
top: 0;
}

.pagelayer-quote-content .fa-quote-right{
display: inline-block;
vertical-align: top;
}

/* Quote end */

.pagelayer-call-icon-section,
.pagelayer-call-content-section,
.pagelayer-call-button-section{
display: table-cell;
vertical-align: middle;	
}

.pagelayer-call-center{
text-align: center;
}

.pagelayer-call-center .pagelayer-call-icon-section,
.pagelayer-call-center .pagelayer-call-content-section,
.pagelayer-call-center .pagelayer-call-button-section{
display: block;
}

.pagelayer-call-3d{
background: #eee;
border-bottom: 5px solid #ddd;
padding: 30px;
}

.pagelayer-call-3d.pagelayer-call-left .pagelayer-call-icon-section{
padding-right: 10px;
}

.pagelayer-call-3d.pagelayer-call-left .pagelayer-call-content-section{
padding-right: 20px
}

.pagelayer-call-3d.pagelayer-call-center .pagelayer-call-icon-section{
margin-bottom: 10px;
}

.pagelayer-call-3d.pagelayer-call-center .pagelayer-call-content-section{
margin-bottom: 20px;	
}

.pagelayer-call-3d .pagelayer-call-title{
font-size: 26px;
font-weight: 700;
/* margin-bottom: 5px; */
}
.pagelayer-call-3d .pagelayer-call-subtitle{
font-size: 18px;
/* margin-bottom: 10px; */
}

/* .pagelayer-call-3d .pagelayer-call-text{
color: #777;
} */

.pagelayer-call-left .pagelayer-call-button{
display:flex;
}

.pagelayer-button-mini{
font-size: 13px;
padding: 8px 12px;
line-height: 13px;
min-height: unset;
}

.pagelayer-button-small{
font-size: 15px;
padding: 10px 18px;
line-height: 15px;
min-height: unset;	
}

.pagelayer-button-middle{
font-size: 17px;
padding: 14px 26px;
line-height: 17px;
min-height: unset;	
}

.pagelayer-button-large{
font-size: 20px;
padding: 18px 35px;
line-height: 20px;
min-height: unset;	
}


.pagelayer-modal-content, 
.pagelayer-splash-container{
display: none;
position: fixed;
padding: 50px;
top: 0;
left: 0;
right: 0;
height: 100%;
overflow: auto;
color: #000;
opacity: 1;
z-index: 99999;
box-sizing: border-box;
}

.pagelayer-modal-bottom-content,
.pagelayer-splash-bottom-content{
padding: 50px;
background-color: #fefefe;
}
.pagelayer-modal-body,
.pagelayer-splash-body{
position: absolute;
margin: auto;
box-shadow: 0 4px 8px 0 rgba(0,0,0,0.2),0 6px 20px 0 rgba(0,0,0,0.19);
-webkit-animation-name: animatetop;
-webkit-animation-duration: 0.4s;
animation-name: animatetop;
animation-duration: 0.4s;
left: 50%;
top: 50%;
transform: translate(-50%, -50%);
}

.pagelayer-modal-content-overflow, .pagelayer-splash-content-overflow{
max-height:500px;
overflow:auto;
}

.pagelayer-splash-bg-close, .pagelayer-modal-bg-close{
position: absolute;
top:0;
left:0;
width:100%;
height:100%;
}

.pagelayer-btn-icon-left .pagelayer-icon-right{
display: none;
}

.pagelayer-btn-icon-right .pagelayer-icon-left{
display: none;
}

/* Add Animation */
@-webkit-keyframes pagelayer-animatetop {
from {top:-300px; opacity:0} 
to {top:0; opacity:1}
}

@keyframes pagelayer-animatetop {
from {top:-300px; opacity:0}
to {top:0; opacity:1}
}

.pagelayer-modal-close,
.pagelayer-splash-close{
position: absolute;
right: 10px;
top: 25px;
color: #fff;
font-size: 70px !important;
cursor: pointer;
}

.pagelayer-modal-close:before,
.pagelayer-modal-close:after,
.pagelayer-splash-close:before,
.pagelayer-splash-close:after{
position: absolute;
top: 2px;
right: 38px;
content: ' ';
width: 2px;
background-color: #fff;
}

.pagelayer-modal-close:before,
.pagelayer-splash-close:before{
-webkit-transform: rotate(45deg);
-ms-transform: rotate(45deg);
transform: rotate(45deg);
}

.pagelayer-modal-close:after,
.pagelayer-splash-close:after{
-webkit-transform: rotate(-45deg);
-ms-transform: rotate(-45deg);
transform: rotate(-45deg);
}

.pagelayer-modal-title,
.pagelayer-splash-title{
background-color: #3D54DF;
color: #ffffff;
}


/* Countdown Start */
.pagelayer-countdown-item{
padding:20px 30px;
text-align:center;
}

.pagelayer-countdown-counter{
display: flex;
flex-wrap:wrap;
justify-content: center;
}

.pagelayer-countdown-days,
.pagelayer-countdown-hours,
.pagelayer-countdown-minutes,
.pagelayer-countdown-seconds{
display: inline-block;
vertical-align: middle;
background: #eee;
margin: 0 5px 5px;
flex:1;
}

.pagelayer-countdown-days div,
.pagelayer-countdown-hours div,
.pagelayer-countdown-minutes div,
.pagelayer-countdown-seconds div{
/* display: inline-block; */
line-height: 1;
}

.pagelayer-countdown-inline .pagelayer-countdown-name{
display:inline-block;
}

.pagelayer-countdown-inline .pagelayer-countdown-count{
display:inline-block;
}

.pagelayer-countdown-expired{
display:none;
}

.pagelayer-countdown[display_expired_text="true"] .pagelayer-countdown-expired{
display:block !important;
}

.pagelayer-countdown[display_expired_text="true"] .pagelayer-countdown-counter{
display:none !important;
}
/* Countdown End*/

#pagelayer-header-menu{
display: inline-block;
}


/* splash style */
.pagelayer-splash .pagelayer-splash-dark,
.pagelayer-modal .pagelayer-modal-dark{
background-color:#000000e0;
}

.pagelayer-modal .pagelayer-modal-light,
.pagelayer-splash .pagelayer-splash-light{
background-color:#ffffffe0;
}

.pagelayer-splash-light .pagelayer-splash-close::after,
.pagelayer-splash-light .pagelayer-splash-close::before,
.pagelayer-modal-light .pagelayer-modal-close::after,
.pagelayer-modal-light .pagelayer-modal-close::before{
background-color:#000000;
}

.pagelayer-splash-dark .pagelayer-splash-close::after,
.pagelayer-splash-dark .pagelayer-splash-close::before,
.pagelayer-modal-dark .pagelayer-modal-close::after,
.pagelayer-modal-dark .pagelayer-modal-close::before{
background-color:#fff;
}

.pagelayer-splash-content{
padding:30px;
}

/* splash style end */

.pagelayer-modal-close{
z-index:999;
}

/* Style the tab */
.pagelayer-tabs-holder {
overflow: hidden;
}

/* Style the buttons inside the tab */
.pagelayer-tabs-holder .pagelayer-tablinks {
background-color: inherit;
display:inline-block;
border: none;
outline: none;
cursor: pointer;
padding: 14px 16px;
transition: 0.3s;
font-size: 17px;
color:#000000;
}

/* Change background color of buttons on hover */
.pagelayer-tabs-holder .pagelayer-tablinks:hover {
background-color: #ddd;
}

/* Create an active/current tablink class */
.pagelayer-tabs-holder .pagelayer-tablinks.active {
background-color: #ccc;
}

/* Style the tab content */
.pagelayer-tabs .pagelayer-tabcontainer .pagelayer-tab{
display: none;
padding: 6px 12px;
border-top: none;
}
.pagelayer-tabs-holder .pagelayer-tablinks .fa{
line-height:1.5;
}
/* Tabs style end */

/* Divider start */
.pagelayer-divider-holder{
line-height: 0;
font-size: 0;
}
.pagelayer-divider-seperator{
display: inline-block;
}
/* Divider end */

/* Counter style start*/

.pagelayer-counter{
padding: 20px;	
}

.pagelayer-counter-content{
line-height:1;
}

/* Counter style end*/

/*Image Slider style start*/
.pagelayer-image-slider-ul{
display:none;
padding:0;
margin:0;
list-style: none;
}

.pagelayer-image-slider-ul[pagelayer-setup]{
display:block;
}
/*Image Slider style End*/

/*Google Maps start*/
.pagelayer-google-maps-holder{
line-height:0;
}

.pagelayer-google-maps-holder iframe{
margin:0px;
width:100%;
height:100%;
}

/* accordion style */
.pagelayer-accordion_item{
overflow:hidden;
}

.pagelayer-accordion-tabs {
cursor: pointer;
padding: 15px;
width: 100%;
text-align: left;
display:inline-block;
text-decoration:none !important;
}

.pagelayer-accordion-panel {
padding: 0 18px;
display: none;
overflow: hidden;
}

/* accordion style end */

/* Alert Box style start */
.pagelayer-alert{
position: relative;
border: 1px solid transparent;
border-radius:5px;
padding:15px 20px;
}

.pagelayer-alert-title{
display:inline-block;
}

.pagelayer-alert-icon,
.pagelayer-alert-title{
vertical-align: middle;
}

.pagelayer-alert-primary{
color: #004085;
background-color: #cce5ff;
border-color: #b8daff;
}

.pagelayer-alert-secondary{
color: #383d41;
background-color: #e2e3e5;
border-color: #d6d8db;
}

.pagelayer-alert-success{
color: #155724;
background-color: #d4edda;
border-color: #c3e6cb;
}

.pagelayer-alert-info{
color: #0c5460;
background-color: #d1ecf1;
border-color: #bee5eb;
}

.pagelayer-alert-warning{
color: #856404;
background-color: #fff3cd;
border-color: #ffeeba;
}

.pagelayer-alert-danger{
color: #721c24;
background-color: #f8d7da;
border-color: #f5c6cb;
}

.pagelayer-alert-dark{
color: #1b1e21;
background-color: #d6d8d9;
border-color: #c6c8ca;
}

.pagelayer-alert-primary-link {
color: #002752;
}

.pagelayer-alert-secondary-link{
color: #202326;
}

.pagelayer-alert-success-link{
color: #0b2e13;
}

.pagelayer-alert-danger-link{
color: #491217;
}

.pagelayer-alert-warning-link{
color: #533f03;
}

.pagelayer-alert-info-link{
color: #062c33;
}

.pagelayer-alert-secondary-link{
color: #686868;
}

.pagelayer-alert-secondary-link{
color: #040505;
}

.pagelayer-alert-success hr{
background-color: #b1dfbb;	
}

.pagelayer-alert-primary hr{
background-color: #9fcdff;
}

.pagelayer-alert-secondary hr{
background-color: #c8cbcf;
}

.pagelayer-alert-info hr{
background-color: #abdde5;
}

.pagelayer-alert-warning hr{
background-color: #ffe8a1;
}

.pagelayer-alert-danger hr{
background-color: #f1b0b7;
}

.pagelayer-alert-dark hr{
background-color: #b9bbbe;
}

.pagelayer-alert-close {
position: absolute;
top: 0;
right: 0;
color: inherit;
height: 100%;
width:15px;
background:rgba(3,3,3,0.1);
cursor:pointer;
}

.pagelayer-alert-close:before,
.pagelayer-alert-close:after {
position: absolute;
left: 7px;
content: ' ';
height: 13px;
width: 1px;
background-color: #333;
top: calc(50% - 6.5px);
}

.pagelayer-alert-close:before {
transform: rotate(45deg);
}

.pagelayer-alert-close:after {
transform: rotate(-45deg);
}

/* Alert Box Style Ends */

/* Grid Gallery Style Start */

.pagelayer-grid-gallery-ul{
display:block;
list-style:none;
padding:0;
margin:0;
}

.pagelayer-gallery-item{
list-style:none;
border: none;
}

.pagelayer-gallery-item img{
height: 100%;
width: 100%;
object-fit: cover;
}

.pagelayer-grid-gallery-caption{
display:block;
}

.pagelayer-grid-gallery-pagination{
text-align:center;
}

.pagelayer-grid-page-ul{
display: inline-block;
list-style-type: none;
margin-top:10px;
}

.pagelayer-grid-page-ul li.active {
background-color:#00A0D2;
color:white;
}

.pagelayer-grid-page-item{
color: black;
float: left;
padding: 8px 16px;
text-decoration: none;
}

.pagelayer-grid-page-ul li:hover:not(.active){
background-color: #ddd;
cursor:pointer;
}

/* Grid Gallery Style Ends */

.pagelayer-heading-holder *,
.pagelayer-text-holder *{
padding: 0;
margin: 0;
overflow-wrap: break-word;
}

/* animation */

.animated.pagelayer-anim-fast{
-webkit-animation-duration: 0.7s;
animation-duration: 0.7s;
}

.animated.pagelayer-anim-fastest{
-webkit-animation-duration: 0.5s;
animation-duration: 0.5s;
}

.animated.pagelayer-anim-slow{
-webkit-animation-duration: 1.5s;
animation-duration: 1.5s;
}

.animated.pagelayer-anim-slowest{
-webkit-animation-duration: 2s;
animation-duration: 2s;
}

/* animation end */

.pagelayer-parallax-window{
position: absolute;
width: 100%;
height: 100%;
overflow: hidden;
top: 0;
left: 0;
}

.pagelayer-parallax-window .simpleParallax{
height:100%;
}

.pagelayer-parallax-window img{
max-width:unset;
}

/* WooCommerce widget */
.pagelayer-product-images-container{
display:flow-root;
}

.pagelayer-add-to-cart-holder,
.pagelayer-product-rating{
display:inline-block;
}

.pagelayer-product-related-container:not([pagelayer-heading-show]) .products > h2,
.pagelayer-product-related-container:not([pagelayer-sale-flash]) ul.products li.product span.onsale,
.pagelayer-addi-info-container:not([pagelayer-show-heading]) h2 {
display:none;
}

.pagelayer-product-related-container[pagelayer-content-align="right"] ul.products li.product .star-rating{
margin-left: auto;
}

.pagelayer-product-related-container[pagelayer-content-align="center"] ul.products li.product .star-rating{
margin-left: auto;
margin-right: auto;
}

/*******************/

/* Pagelayer Owl */

.pagelayer-owl-stage-outer [class^="pagelayer-owl-"],
.pagelayer-owl-carousel .pagelayer-owl-item > .pagelayer-ele-wrap,
.pagelayer-owl-stage-outer{
height: 100%;
}

.pagelayer-owl-carousel .pagelayer-owl-item{
height: auto;
}

.pagelayer-owl-prev,
.pagelayer-owl-next{
position: absolute;
top: 50%;
transform: translateY(-50%);
}

.pagelayer-owl-prev{
left: 0;
}

.pagelayer-owl-next{
right: 0;
}

.pagelayer-owl-theme .pagelayer-owl-nav{
margin-top: 0 !important;
}

.pagelayer-owl-nav span{
position:absolute;
transform:translate(-50%, -53%);
}

.pagelayer-owl-dot{
vertical-align: middle;
}

/* Pagelayer Owl End */

/* Space Widget */
.pagelayer-space-holder{
height: 10px;
}
/* Space Widget End */

/* Address and Number */

.pagelayer-phone-holder,
.pagelayer-address-holder,
.pagelayer-email-holder{
display:flex;
}

.pagelayer-address-icon,
.pagelayer-address,
.pagelayer-phone-icon,
.pagelayer-phone,
.pagelayer-email-icon,
.pagelayer-email{
margin-top:auto;
margin-bottom:auto;
word-break:break-word;
}

/* Address and Number End */

/****************/
/*** Freemium ***/
/****************/

/*** Breadcrumb ***/

.pagelayer-breadcrumb-section b{
font-weight: 100;
}

/* Breadcrumb End */

/*** Archive Posts ***/
.pagelayer-posts-container{
display: grid;
grid-template-columns: repeat(3,1fr);
grid-column-gap: 20px;
grid-row-gap: 40px;
}

.pagelayer-wposts-meta *{
font-size: 12px;
}

.pagelayer-pagination{
padding: 50px 20px;	
text-align: center;
}

.pagelayer-pagination .page-numbers:not(:last-child){
margin-right: 25px;
}

.pagelayer-pagination a.page-numbers:hover{
color: #36b2d1;
}

.pagelayer-wposts-thumb{
display:inline-block;
position:relative;
background-size:cover !important;
background-position:center !important;
width:100%;
}

@media all and (max-width:599px){	
.pagelayer-posts-container{
grid-template-columns: repeat(1,1fr);
}
}

/*** Archive Posts End ***/

/* WordPress Posts */
.pagelayer-wposts-title{
line-height:1;
}

.pagelayer-wposts-sep{
font-weight:100;
}

.pagelayer-wposts-sep:last-child{
display:none;
}

.pagelayer-wposts-category a:not(:last-child):after,
.pagelayer-wposts-tags a:not(:last-child):after{
content:', ';	
}
/* WordPress Posts End*/

/* Copyright start */
.pagelayer-copyright{
text-align: center;
color: #111;
}

.pagelayer-copyright a{
color: #111;
}
/* Copyright end */

/* Primary Menu */

.pagelayer-wp-menu-container .pagelayer-wp_menu-ul li.menu-item>a{
display:flex;
box-shadow: none;
white-space: nowrap;
position:relative;
}

.pagelayer-menu-type-horizontal .sub-menu{
z-index:999;
}

.pagelayer-wp-menu-container .pagelayer-wp_menu-ul,
.pagelayer-wp-menu-container .sub-menu{
margin:0px;
padding: 0px;
}

.pagelayer-menu-type-horizontal *{
transition: all 0.5s;	
}

.pagelayer-menu-type-horizontal li.menu-item{
display: inline-block;
font-weight: 600;
}

.pagelayer-menu-type-horizontal li.menu-item>a{
box-shadow: none;
border: none;
}

.pagelayer-menu-type-horizontal .sub-menu{
position:absolute;
min-width: 100%;
}

.pagelayer-menu-type-horizontal .sub-menu .sub-menu{
top:0px;
}

.pagelayer-wp-menu-container .pagelayer-wp_menu-ul li.menu-item{
position:relative;
}

.pagelayer-wp-menu-container ul li.menu-item span.after-icon{
height: auto;
width: 100%;
position: relative;
pointer-events: auto;
line-height:inherit;
color: inherit;
font-size: inherit;
margin-left:10px;
padding-left:5px;
padding-right:5px;
}

.pagelayer-wp-menu-container:not([data-align="left"]) .pagelayer-wp_menu-ul span.after-icon{
width:auto;
}

.pagelayer-wp-menu-container[data-align="right"] .pagelayer-wp_menu-ul ul li > ul a{justify-content: flex-end;}
.pagelayer-wp-menu-container[data-align="left"] .pagelayer-wp_menu-ul ul li > ul a{justify-content: left;}
.pagelayer-wp-menu-container[data-align="center"] .pagelayer-wp_menu-ul ul li > ul a{justify-content: center;}

.pagelayer-wp-menu-container ul li.menu-item span.after-icon:before{
position:absolute;
right:0;	
}

.pagelayer-wp-menu-container .sub-menu,
.pagelayer-primary-menu-bar{
display:none;
}

.pagelayer-menu-type-horizontal .sub-menu li.menu-item{
display:block;
}

.pagelayer-primary-menu-bar i{
padding:5px;
}

.pagelayer-menu-type-vertical li.menu-item{
list-style: none;
}

.pagelayer-menu-type-vertical li.menu-item:not(:last-child) > a{
margin-bottom: 6px !important;
box-shadow: none;
border: none;
}

.pagelayer-menu-type-vertical .pagelayer-wp_menu-ul>li>ul.sub-menu{
margin-left:0px;
}

.pagelayer-wp-menu-holder[data-layout="dropdown"] .pagelayer-primary-menu-bar,
.pagelayer-wp-menu-holder.pagelayer-wp-menu-dropdown .pagelayer-primary-menu-bar{
display:block;
}

.pagelayer-wp-menu-holder[data-layout="dropdown"] .pagelayer-wp_menu-ul,
.pagelayer-wp-menu-holder.pagelayer-wp-menu-dropdown .pagelayer-wp_menu-ul{
display:none;	
}

.pagelayer-menu-hover-underline:not(.none) .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-framed:not(.none) .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-doubleline:not(.none) .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-doubleline:not(.none) .pagelayer-wp_menu-ul>li>a:after,
.pagelayer-menu-hover-overline:not(.none) .pagelayer-wp_menu-ul>li>a:before{
position: absolute;
content: '';
left: 0px;
bottom: 0px;
height: 3px;
width: 0px;
background-color: #2154CF;
transition: all 500ms ease;
-moz-transition: all 500ms ease;
-webkit-transition: all 500ms ease;
-ms-transition: all 500ms ease;
-o-transition: all 500ms ease;
border-width:0px;
}

.pagelayer-menu-hover-doubleline .pagelayer-wp_menu-ul>li>a:after{
left:unset;
right:0;
}

.pagelayer-menu-hover-framed .pagelayer-wp_menu-ul>li>a:before{
background-color:unset !important;
height:100% !important;
}
.pagelayer-menu-hover-framed .pagelayer-wp_menu-ul>li>a:hover:before{
border:3px solid #2154CF;	
}

.pagelayer-menu-hover-overline .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-doubleline .pagelayer-wp_menu-ul>li>a:before{
top:0;
bottom:unset;
}

.pagelayer-menu-hover-underline.dropin .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-framed.dropin .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-doubleline.dropin .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-doubleline.dropin .pagelayer-wp_menu-ul>li>a:after,
.pagelayer-menu-hover-overline.dropin .pagelayer-wp_menu-ul>li>a:before{
width:100%;
opacity:0;
bottom:-5px;
} 

.pagelayer-menu-hover-underline.dropout .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-framed.dropout .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-doubleline.dropout .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-doubleline.dropout .pagelayer-wp_menu-ul>li>a:after,
.pagelayer-menu-hover-overline.dropout .pagelayer-wp_menu-ul>li>a:before{
width:100%;
opacity:0;
bottom:5px;
} 

.pagelayer-menu-hover-underline.dropin .pagelayer-wp_menu-ul>li>a:hover:before,
.pagelayer-menu-hover-framed.dropin .pagelayer-wp_menu-ul>li>a:hover:before,
.pagelayer-menu-hover-doubleline.dropin .pagelayer-wp_menu-ul>li>a:hover:before,
.pagelayer-menu-hover-doubleline.dropin .pagelayer-wp_menu-ul>li>a:hover:after,
.pagelayer-menu-hover-overline.dropin .pagelayer-wp_menu-ul>li>a:hover:before,
.pagelayer-menu-hover-underline.dropout .pagelayer-wp_menu-ul>li>a:hover:before,
.pagelayer-menu-hover-framed.dropout .pagelayer-wp_menu-ul>li>a:hover:before,
.pagelayer-menu-hover-doubleline.dropout .pagelayer-wp_menu-ul>li>a:hover:before,
.pagelayer-menu-hover-doubleline.dropout .pagelayer-wp_menu-ul>li>a:hover:after,
.pagelayer-menu-hover-overline.dropout .pagelayer-wp_menu-ul>li>a:hover:before{
bottom:0px;
} 

.pagelayer-menu-hover-underline.grow .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-framed.grow .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-doubleline.grow .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-doubleline.grow .pagelayer-wp_menu-ul>li>a:after,
.pagelayer-menu-hover-overline.grow .pagelayer-wp_menu-ul>li>a:before{
width:100%;
transform:scale(0);
}

.pagelayer-menu-hover-underline.fade .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-framed.fade .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-doubleline.fade .pagelayer-wp_menu-ul>li>a:before,
.pagelayer-menu-hover-doubleline.fade .pagelayer-wp_menu-ul>li>a:after,
.pagelayer-menu-hover-overline.fade .pagelayer-wp_menu-ul>li>a:before{
width:100%;
transition-duration:1000ms;
opacity:0;
}

.pagelayer-menu-hover-underline .pagelayer-wp_menu-ul>li>a:hover:before,
.pagelayer-menu-hover-framed .pagelayer-wp_menu-ul>li>a:hover:before,
.pagelayer-menu-hover-doubleline .pagelayer-wp_menu-ul>li>a:hover:before,
.pagelayer-menu-hover-doubleline .pagelayer-wp_menu-ul>li>a:hover:after,
.pagelayer-menu-hover-overline .pagelayer-wp_menu-ul>li>a:hover:before{
transform:scale(1);
opacity:1;
width:100%;
}

.pagelayer-menu-hover-text .pagelayer-wp_menu-ul>li>a:hover{
transform:scale(1.2);
}

.pagelayer-wp_menu .sub-menu,
.pagelayer-menu-type-dropdown{
z-index:999;
}

.pagelayer-menu-type-dropdown{
position:absolute;
}

.pagelayer-togglt-on .pagelayer-wp_menu-ul,
.pagelayer-active-sub-menu:not(.pagelayer-mega-menu-item) > ul.sub-menu,
.pagelayer-menu-type-horizontal .menu-item-has-children:not(.pagelayer-mega-menu-item):hover > ul.sub-menu,
.pagelayer-wp_menu .pagelayer-menu-type-vertical[class*='pagelayer-submenu-position-'] .menu-item-has-children:not(.pagelayer-mega-menu-item):hover > ul.sub-menu{
display:block !important;
}

.pagelayer-menu-type-dropdown.pagelayer-wp_menu-ul,
.pagelayer-menu-type-dropdown.pagelayer-wp_menu-right,
.pagelayer-menu-type-dropdown.pagelayer-wp_menu-left,
.pagelayer-menu-type-dropdown.pagelayer-wp_menu-full{
position: fixed;
z-index: 999;
height: 100%;
top: 0;
transition: all 0.4s;
}

.pagelayer-menu-type-dropdown.pagelayer-wp_menu-right{
right: -100%;	
}

.pagelayer-menu-type-dropdown.pagelayer-wp_menu-left{
left:-100%;
}

.pagelayer-wp_menu-left.pagelayer-togglt-on{
left:0;
}

.pagelayer-wp_menu-right.pagelayer-togglt-on{
right:0;	
}

.pagelayer-menu-type-dropdown.pagelayer-wp_menu-full{
left:0;
right:0;
top:-100%;
}

.pagelayer-wp_menu-full.pagelayer-togglt-on{
top:0;
}

.pagelayer-wp_menu-close{
cursor:pointer;
}

.pagelayer-menu-type-dropdown .pagelayer-wp_menu-close{
display:block;
}

.pagelayer-wp_menu-close,
.pagelayer-wp_menu-down .pagelayer-wp_menu-close{
display:none;
}

.pagelayer-wp_menu-close i{
position:absolute;
z-index:99;
}

.pagelayer-menu-type-dropdown.pagelayer-wp_menu-right .pagelayer-wp_menu-ul,
.pagelayer-menu-type-dropdown.pagelayer-wp_menu-left .pagelayer-wp_menu-ul,
.pagelayer-menu-type-dropdown.pagelayer-wp_menu-full .pagelayer-wp_menu-ul{
position:absolute;
width:100%;
}

.pagelayer-menu-type-dropdown .pagelayer-wp_menu-ul li.menu-item > a{
white-space:normal !important;
}

.pagelayer-menu-type-dropdown .pagelayer-wp_menu-ul{
height:100%;
max-height: -webkit-fill-available;
overflow-y: scroll;
-webkit-overflow-scrolling: touch;
}

.pagelayer-menu-type-dropdown .pagelayer-wp_menu-ul::-webkit-scrollbar {
width:4px;
height:4px;
}

.pagelayer-menu-type-dropdown .pagelayer-wp_menu-ul::-webkit-scrollbar-track  {
background-color: transparent;
}

.pagelayer-menu-type-dropdown .pagelayer-wp_menu-ul::-webkit-scrollbar-thumb {
box-shadow: inset 0 0 6px rgba(0, 0, 0, 0.3);
border-radius:10px;
}
/* Mega menu css start */

.pagelayer-wp_menu-ul .pagelayer-mega-menu{
transition: none;
position: absolute;
max-width: 100vw;
z-index: 999;
padding: 10px;
background: #fff;
color: #000;
display: none;
border-radius: 2px;
left: 0;
text-align: initial;
overflow-y: auto;
}

.pagelayer-wp_menu .pagelayer-menu-type-vertical .pagelayer-mega-menu-item,
.pagelayer-wp-menu-container:not(.pagelayer-menu-type-horizontal) .menu-item > .pagelayer-mega-menu{
position: relative;
width: 100%;
}

.pagelayer-wp_menu-ul .pagelayer-mega-menu *{
transition: none;
}

/* To avoid row widget full width feature */
.pagelayer-wp_menu-ul .pagelayer-mega-menu .pagelayer-wrap-inner-row,
.pagelayer-wp_menu-ul .pagelayer-mega-menu .pagelayer-inner_row{
max-width: 100% !important;
left: auto !important;
}

.pagelayer-wp_menu-ul .pagelayer-set-position{
display: block !important;
visibility: hidden !important;
opacity: 0 !important;
}

.pagelayer-active-sub-menu.pagelayer-mega-menu-item > .pagelayer-mega-menu,
.pagelayer-active-mega-menu.pagelayer-mega-menu-item > .pagelayer-mega-menu,
:not(.pagelayer-ele-wrap) > .pagelayer-wp_menu .pagelayer-menu-type-horizontal .pagelayer-mega-menu-item:hover > .pagelayer-mega-menu,
:not(.pagelayer-ele-wrap) > .pagelayer-wp_menu .pagelayer-menu-type-vertical[class*='pagelayer-submenu-position-'] .pagelayer-mega-menu-item:hover > .pagelayer-mega-menu{
display: block !important;
}

@keyframes pagelayer-submenu-fade{0%{opacity:0}75%{opacity:1}}
@keyframes pagelayer-submenu-pulse{50%{transform: scale3d(1.04, 1.04, 1.04);}100%{transform: scale3d(1, 1, 1);}}
@keyframes pagelayer-submenu-fadeindown{0%{opacity: 0; transform: translate3d(0, -10px, 0); }100%{ opacity: 1; transform: none; }}
@keyframes pagelayer-submenu-fadeinup{0%{opacity: 0;transform: translate3d(0, 20px, 0);}100%{opacity: 1;transform: none;}}
@keyframes pagelayer-submenu-slideindown{0%{transform: translate3d(0, -10px, 0);visibility: visible;}100%{transform: translate3d(0, 0, 0);}}
@keyframes pagelayer-submenu-slideinup{0%{transform: translate3d(0, 20px, 0);visibility: visible;}100%{transform: translate3d(0, 0, 0);}}
@keyframes pagelayer-submenu-zoomin{0%{opacity: 0;transform: scale3d(.5, .5, .5);}50%{opacity: 1;}}
@keyframes pagelayer-submenu-flip{
0%{
transform: perspective(400px) rotate3d(1, 0, 0,90deg);
animation-timing-function: ease-in;
opacity: 0;
}40%{
transform: perspective(400px) rotate3d(1, 0, 0, -5deg);
animation-timing-function: ease-in;
}60%{
transform: perspective(400px) rotate3d(1, 0, 0, 5deg);
opacity: 1;
}80%{
transform: perspective(400px) rotate3d(1, 0, 0, -5deg);
}100% {
transform: perspective(400px);
}
}

.pagelayer-wp-menu-container.pagelayer-menu-type-vertical.pagelayer-submenu-position-right .pagelayer-mega-menu,
.pagelayer-menu-type-vertical.pagelayer-submenu-position-right .sub-menu{
position: absolute;
left: 100%;
top: 0;
right: auto;
}

.pagelayer-wp-menu-container.pagelayer-menu-type-vertical.pagelayer-submenu-position-left .pagelayer-mega-menu,
.pagelayer-menu-type-vertical.pagelayer-submenu-position-left .sub-menu{
position: absolute;
right: 100%;
top: 0;
left: auto;
}

.pagelayer-menu-type-vertical:not(.pagelayer-submenu-position-right, .pagelayer-submenu-position-left) .pagelayer-mega-menu,
.pagelayer-menu-type-vertical:not(.pagelayer-submenu-position-right, .pagelayer-submenu-position-left) .sub-menu{
width:100% !important;
}

.pagelayer-menu-icon{
align-self: center;
text-align: center;
margin: 0 5px 0 0;
}

.pagelayer-nav-menu-icon-right .pagelayer-menu-icon{
margin: 0 0 0 5px;
order:1;
}

.pagelayer-nav-menu-icon-right .pagelayer-menu-icon ~ .pagelayer-nav-menu-title{
order: 0;
}

.pagelayer-nav-menu-icon-right .pagelayer-menu-icon ~ :not(.pagelayer-nav-menu-title, .pagelayer-menu-icon){
order: 2;
}

.pagelayer-menu-highlight{
font-size: 0.7em;
margin-left: 5px;
padding: 3px 6px;
border-radius: 2px;
}

/* Mega menu css ends */

/* Column Menu css*/
.pagelayer-mega-column-item > .sub-menu{
columns: 2;
}

.pagelayer-mega-column-item .sub-menu .sub-menu .pagelayer-nav-menu-title{
font-size: 0.85em;
}

.pagelayer-mega-column-item .sub-menu .sub-menu{
display: block;
position: static;
}

.pagelayer-mega-column-item .sub-menu li.menu-item{
break-inside: avoid;
}

.pagelayer-mega-column-item ul.sub-menu span.after-icon{
display: none;
}
/* Primary Menu End */

/* Contact Form start */
.pagelayer-contact-form-note{
margin-bottom:10px;
}

.pagelayer-contact_item input,
.pagelayer-contact_item textarea,
.pagelayer-contact_item select{
width:100%;
outline:none;
}

.pagelayer-contact_item textarea{
height:auto;
}

.pagelayer-contact-holder input[type="checkbox"] {
visibility: hidden;
display: contents;
}

.pagelayer-contact-holder label {
cursor: pointer;
display: block;
}

.pagelayer-contact-holder input[type="checkbox"] + label:before {
border: 1px solid #333;
content: "\00a0";
display: inline-block;
font: 16px/1em sans-serif;
margin-right: 0.25em;
padding: 0;
vertical-align: middle;
}

.pagelayer-contact-holder input[type="checkbox"]:checked + label:before {
content: "\2713";
text-align: center;
}

.pagelayer-contact-holder input[type="checkbox"]:checked + label:after {
font-weight: bold;
}

.pagelayer-contact-holder input[type="checkbox"]:focus + label::before {
outline: rgb(59, 153, 252) auto 5px;
}

.pagelayer-contact-holder input[type='radio'] {
-webkit-appearance:none;
border-radius:50%;
outline:none;
vertical-align: middle;
box-shadow:0 0 5px 0px gray inset;
padding: 0 !important;
}

.pagelayer-contact-holder input[type="radio"]:checked:before {
  background: #333333;
}

.pagelayer-contact-holder input[type='radio']:hover {
box-shadow:0 0 5px 0px orange inset;
}

.pagelayer-contact-holder input[type='radio']:before {
content:'';
display:block;
width:60%;
height:60%;
margin: 20% auto;    
border-radius:50%;    
}

.pagelayer-contact-submit-btn{
cursor:pointer;
}

.pagelayer-cf-msg-suc,.pagelayer-cf-msg-err{
padding: 10px;
padding: 10px;
margin: 10px 0px;
background: #a4f4ad;
border: 1px solid #6fc16f;
border-radius: 4px;
}

.pagelayer-cf-msg-err{
background: #f9dacb;
border: 1px solid #ff746e;
}
/* Contact Form End */

/*** Post Excerpt ***/

.pagelayer-empty-widget{
height:30px;
width:100%;
background-color: #dedddd;
text-align:center;
}

.pagelayer-empty-widget:after {
font-family: "FontAwesome" , "Font Awesome 5 Free";
font-weight: 900;
}

.pagelayer-post-excerpt.pagelayer-empty-widget:after{
content: '\f15c';	
}

.pagelayer-featured-img.pagelayer-empty-widget:after{
content: '\f03e';
}

/*** Post Excerpt End ***/

/* Post info*/
.pagelayer-post-info-vertical > div{
display:inline-block;
}
/* Post info end*/


/* Post Nav */
.pagelayer-post-nav-container{
display: flex;
align-items: center;
}

.pagelayer-prev-post a,
.pagelayer-next-post a{
display: inline-flex;
align-items: center;
}

.pagelayer-next-post a .pagelayer-post-nav-icon{
padding-left:12px;
}

.pagelayer-prev-post a .pagelayer-post-nav-icon{
padding-right:12px;
}

.pagelayer-next-post{
text-align:right;
}

.pagelayer-prev-post a:hover,
.pagelayer-next-post a:hover{
text-decoration:none;
}

.pagelayer-prev-post, .pagelayer-next-post{
width: calc(50% - 1px);
}

.pagelayer-post-nav-container .pagelayer-prev-holder,
.pagelayer-post-nav-container .pagelayer-next-holder{
display: -webkit-box;
display: -webkit-flex;
display: -ms-flexbox;
display: flex;
-webkit-box-orient: vertical;
-webkit-box-direction: normal;
-webkit-flex-direction: column;
-ms-flex-direction: column;
flex-direction: column;
}

.pagelayer-post-nav-separator{
align-self: stretch;
}
/* Post Nav end */

/*Flipbox css start*/
.pagelayer-flipbox-container{
margin:0 auto;
}

.pagelayer-flipbox-flipper{
position: relative;
height: 300px;
perspective: 1000px;
-webkit-perspective: 1000px;
transform-style: preserve-3d;
-webkit-transform-style: preserve-3d;
}

.pagelayer-flipbox-front{
background-color: #bc1a1a;
border-radius: inherit;
}

.pagelayer-flipbox-back{
background-color: #f9e73f;
display: block;
border-radius: inherit;
}

.pagelayer-flipbox-flipper:hover .pagelayer-flipbox-front,
.pagelayer-flipbox-flipper .pagelayer-flipbox-back{
opacity: 0;
}

.pagelayer-flipbox-flipper:hover .pagelayer-flipbox-back{
opacity:1;
}

.pagelayer-flipbox-box{
position: absolute;
width: 100%;
height: 100%;
transition: all .8s ease-in-out;
-webkit-transition: all .8s ease-in-out;
}

.pagelayer-flipbox-box-overlay{
display: flex;
display: -webkit-box;
display: -webkit-flex;
display: -ms-flexbox;
width: 100%;
height: 100%;
align-items: stretch;
-webkit-align-items: stretch;
-webkit-box-align: stretch;
flex-direction: column;
-webkit-flex-direction: column;
-webkit-box-direction: normal;
text-align: center;
justify-content: center;
-webkit-justify-content: center;
padding: 35px;
-webkit-box-orient: vertical;
-webkit-box-pack: center;
}

.pagelayer-flipbox-3d .pagelayer-flipbox-box-inner{
transform: translateZ(90px) scale(.91);
-webkit-transform: translateZ(90px) scale(.91);
}

.pagelayer-flipbox-3d .pagelayer-flipbox-box-overlay{
transform-style: preserve-3d;
-webkit-transform-style: preserve-3d;
transform: translateZ(.1px);
-webkit-transform: translateZ(.1px);
}

.pagelayer-flipbox-flip .pagelayer-flipbox-flipper{
transform-style: preserve-3d;
-webkit-transform-style: preserve-3d;
perspective: 1000px;
-webkit-perspective: 1000px;
}

.pagelayer-flipbox-flip .pagelayer-flipbox-box{
transform-style: preserve-3d;
-webkit-transform-style: preserve-3d;
backface-visibility: hidden;
-webkit-backface-visibility: hidden;
}

.pagelayer-flipbox-flip .pagelayer-flipbox-front{
-webkit-transform: none;
-ms-transform: none;
transform: none;
z-index: 1
}

.pagelayer-flipbox-flip .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-back{
transform: none;
-webkit-transform: none;
}

.pagelayer-flipbox-flip.pagelayer-flipbox-direction-right .pagelayer-flipbox-back{
transform: rotateX(0) rotateY(-180deg);
-webkit-transform: rotateX(0) rotateY(-180deg); 
}

.pagelayer-flipbox-flip.pagelayer-flipbox-direction-left .pagelayer-flipbox-back,
.pagelayer-flipbox-flip.pagelayer-flipbox-direction-right .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-front{
transform: rotateX(0) rotateY(180deg);
-webkit-transform: rotateX(0) rotateY(180deg);
}

.pagelayer-flipbox-flip.pagelayer-flipbox-direction-left .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-front{
transform: rotateX(0) rotateY(-180deg);
-webkit-transform: rotateX(0) rotateY(-180deg);
}

.pagelayer-flipbox-flip.pagelayer-flipbox-direction-up .pagelayer-flipbox-back{
transform: rotateX(-180deg) rotateY(0);
-webkit-transform: rotateX(-180deg) rotateY(0);  
}

.pagelayer-flipbox-flip.pagelayer-flipbox-direction-down .pagelayer-flipbox-back,
.pagelayer-flipbox-flip.pagelayer-flipbox-direction-up .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-front{
transform: rotateX(180deg) rotateY(0);
-webkit-transform: rotateX(180deg) rotateY(0);  
}

.pagelayer-flipbox-flip.pagelayer-flipbox-direction-down .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-front{
transform: rotateX(-180deg) rotateY(0);
-webkit-transform: rotateX(-180deg) rotateY(0);  
}

.pagelayer-flipbox-push .pagelayer-flipbox-flipper,
.pagelayer-flipbox-slide .pagelayer-flipbox-flipper{
overflow:hidden;
}

.pagelayer-flipbox-push .pagelayer-flipbox-front{
transform: none;
-webkit-transform: none;
}

.pagelayer-flipbox-push.pagelayer-flipbox-direction-right .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-front{
transform:translateX(100%) translateY(0);
-webkit-transform:translateX(100%) translateY(0);
}

.pagelayer-flipbox-push.pagelayer-flipbox-direction-left .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-front{
transform: translateX(-100%) translateY(0);
-webkit-transform: translateX(-100%) translateY(0);
}

.pagelayer-flipbox-push.pagelayer-flipbox-direction-up .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-front{
transform: translateX(0) translateY(-100%);
-webkit-transform: translateX(0) translateY(-100%);
}

.pagelayer-flipbox-push.pagelayer-flipbox-direction-down .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-front{
transform: translateX(0) translateY(100%);
-webkit-transform: translateX(0) translateY(100%);
}

.pagelayer-flipbox-push .pagelayer-flipbox,
.pagelayer-flipbox-slide .pagelayer-flipbox{
overflow: hidden;
}

.pagelayer-flipbox-push .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-back,
.pagelayer-flipbox-slide .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-back{
transform: none;
-webkit-transform: none;  
}

.pagelayer-flipbox-push.pagelayer-flipbox-direction-right .pagelayer-flipbox-back,
.pagelayer-flipbox-slide.pagelayer-flipbox-direction-right .pagelayer-flipbox-back{
transform: translateX(-100%) translateY(0);
-webkit-transform: translateX(-100%) translateY(0);
}

.pagelayer-flipbox-push.pagelayer-flipbox-direction-left .pagelayer-flipbox-back,
.pagelayer-flipbox-slide.pagelayer-flipbox-direction-left .pagelayer-flipbox-back{
transform: translateX(100%) translateY(0);
-webkit-transform: translateX(100%) translateY(0);  
}

.pagelayer-flipbox-push.pagelayer-flipbox-direction-up .pagelayer-flipbox-back,
.pagelayer-flipbox-slide.pagelayer-flipbox-direction-up .pagelayer-flipbox-back{
transform: translateX(0) translateY(100%);
-webkit-transform: translateX(0) translateY(100%);
}

.pagelayer-flipbox-push.pagelayer-flipbox-direction-down .pagelayer-flipbox-back,
.pagelayer-flipbox-slide.pagelayer-flipbox-direction-down .pagelayer-flipbox-back{
transform: translateX(0) translateY(-100%);
-webkit-transform: translateX(0) translateY(-100%);
}

.pagelayer-flipbox-zoom-out .pagelayer-flipbox-flipper .pagelayer-flipbox-front{
transition: opacity .45s, width .1ms, -webkit-transform .8s;
-webkit-transition: opacity .45s, width .1ms, -webkit-transform .8s;  
transition: transform .8s, opacity .45s, width .1ms;
transition: transform .8s, opacity .45s, width .1ms, -webkit-transform .8s;
opacity: 1;
-webkit-transform: scale(1);
-ms-transform: scale(1);
transform: scale(1);
z-index: 1;
width: 100%
}

.pagelayer-flipbox-zoom-out .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-front{
width: 0;
opacity: 0;
transform: scale(.7);
-webkit-transform: scale(.7);
transition: opacity .8s .1s, width .1ms .8s, -webkit-transform .8s;
-webkit-transition: opacity .8s .1s, width .1ms .8s, -webkit-transform .8s;
transition: transform .8s, opacity .8s .1s, width .1ms .8s;
transition: transform .8s, opacity .8s .1s, width .1ms .8s, -webkit-transform .8s
}

.pagelayer-flipbox-zoom-in .pagelayer-flipbox-flipper .pagelayer-flipbox-back{
-webkit-transition: opacity .5s .2s, -webkit-transform .7s;
transition: opacity .5s .2s, -webkit-transform .7s;
transition: transform .7s, opacity .5s .2s;
transition: transform .7s, opacity .5s .2s, -webkit-transform .7s;
opacity: 0;
transform: scale(.7);
-webkit-transform: scale(.7);  
}

.pagelayer-flipbox-zoom-in .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-back{
-webkit-transition: opacity .5s, -webkit-transform .7s;
transition: opacity .5s, -webkit-transform .7s;
transition: transform .7s, opacity .5s;
transition: transform .7s, opacity .5s, -webkit-transform .7s;
opacity: 1;
transform: scale(1);
-webkit-transform: scale(1);  
}

.pagelayer-flipbox-fade .pagelayer-flipbox-flipper .pagelayer-flipbox-back{
opacity: 0
}

.pagelayer-flipbox-fade .pagelayer-flipbox-flipper:hover .pagelayer-flipbox-back{
opacity: 1
}

.pagelayer-flipbox-container.pagelayer-flipbox-flipped .pagelayer-flipbox-main .pagelayer-flipbox-front{
display: none
}

.pagelayer-flipbox-container.pagelayer-flipbox-flipped .pagelayer-flipbox-main .pagelayer-flipbox-back{
transform: none;
-webkit-transform: none;
opacity: 1;
}

.pagelayer-flipbox-image{
width: 100%;
margin: 0 auto;
object-fit: cover; 
}

.pagelayer-flipbox-image img{
width: 50%;
object-fit: cover;
-o-object-fit: cover;
}

.pagelayer-flipbox[back_section="true"] .pagelayer-flipbox-front{
display:none;
}

.pagelayer-flipbox[back_section="true"] .pagelayer-flipbox-back{
transform: rotateX(0) rotateY(0deg) !important;
-webkit-transform: rotateX(0) rotateY(0deg) !important;
opacity: 1 !important;
}

.pagelayer-flipbox-back .pagelayer-service-btn.pagelayer-btn-link{
background-color:transparent;
}

/*Flipbox End*/

/* Animated Heading */

.pagelayer-aheading-holder {
display: inline-block;
position:relative;
}

.pagelayer-animated-heading{
margin: 0;
padding: 0;
line-height: 1.4;
-webkit-background-clip: text;
}

[class*="pagelayer-blobs"]{
display:block;
position:absolute;
mix-blend-mode:color;
animation:pagelayer-blobs 10s ease-in-out infinite alternate;
}

.pagelayer-blobs_1{
width: 9%;
height: 47%;
top: 12%;
left: 4%;
}

.pagelayer-blobs_2{
width: 10%;
height: 50%;
top: 60%;
left: 34%;
}

.pagelayer-blobs_3{
width: 20%;
height: 46%;
top: 10%;
left: 20%;
}

.pagelayer-blobs_4{
width: 30%;
height: 40%;
top: 30%;
left: 70%;
}

.pagelayer-blobs_5{
width: 12%;
height: 40%;
top: 61%;
left: 12%;
}

.pagelayer-blobs_6{
width: 25%;
height: 45%;
top: 5%;
left: 45%;
}

.pagelayer-blobs_7{
width: 32%;
height: 45%;
top: 67%;
left: 46%;
}

.pagelayer-hEffect-none [class*="pagelayer-blobs"]{
display: none;
}

@keyframes pagelayer-blobs{
0%{border-radius:26% 74% 61% 39% / 54% 67% 33% 46%}
10%{border-radius:74% 26% 47% 53% / 68% 46% 54% 32%}
20%{border-radius:48% 52% 30% 70% / 27% 37% 63% 73%}
30%{border-radius:73% 27% 57% 43% / 28% 67% 33% 72%}
40%{border-radius:63% 37% 56% 44% / 25% 28% 72% 75%}
50%{border-radius:39% 61% 70% 30% / 61% 29% 71% 39%}
60%{border-radius:27% 73% 29% 71% / 73% 51% 49% 27%}
70%{border-radius:39% 61% 65% 35% / 74% 65% 35% 26%}
80%{border-radius:55% 45% 37% 63% / 38% 30% 70% 62%}
90%{border-radius:25% 75% 70% 30% / 39% 50% 50% 61%}
100%{border-radius:66% 34% 33% 67% / 65% 73% 27% 35%}
}

.pagelayer-hEffect-blobs,
.pagelayer-hEffect-none,
.pagelayer-heading-rotating{
-webkit-text-fill-color: transparent;
}

.pagelayer-heading-rotating .pagelayer-animated-heading{
display: inline;
}

.pagelayer-words-wrapper {
  display: inline-block;
  position: relative;
  text-align: left;
}
.pagelayer-words-wrapper span {
  display: inline-block;
  position: absolute;
  white-space: nowrap;
  left: 0;
  top: 0;
}
.pagelayer-words-wrapper span.pagelayer-is-visible {
  position: relative;
}

/*** xrotate-1 ***/
.pagelayer-aheading-rotate1 .pagelayer-words-wrapper {
  -webkit-perspective: 300px;
  -moz-perspective: 300px;
  perspective: 300px;
}
.pagelayer-aheading-rotate1 span {
  opacity: 0;
  -webkit-transform-origin: 50% 100%;
  -moz-transform-origin: 50% 100%;
  -ms-transform-origin: 50% 100%;
  -o-transform-origin: 50% 100%;
  transform-origin: 50% 100%;
  -webkit-transform: rotateX(180deg);
  -moz-transform: rotateX(180deg);
  -ms-transform: rotateX(180deg);
  -o-transform: rotateX(180deg);
  transform: rotateX(180deg);
}
.pagelayer-aheading-rotate1 span.pagelayer-is-visible {
  opacity: 1;
  -webkit-transform: rotateX(0deg);
  -moz-transform: rotateX(0deg);
  -ms-transform: rotateX(0deg);
  -o-transform: rotateX(0deg);
  transform: rotateX(0deg);
  -webkit-animation: pagelayer-rotate-1-in 1.2s;
  -moz-animation: pagelayer-rotate-1-in 1.2s;
  animation: pagelayer-rotate-1-in 1.2s;
}
.pagelayer-aheading-rotate1 span.pagelayer-is-hidden {
  -webkit-transform: rotateX(180deg);
  -moz-transform: rotateX(180deg);
  -ms-transform: rotateX(180deg);
  -o-transform: rotateX(180deg);
  transform: rotateX(180deg);
  -webkit-animation: pagelayer-rotate-1-out 1.2s;
  -moz-animation: pagelayer-rotate-1-out 1.2s;
  animation: pagelayer-rotate-1-out 1.2s;
}

@-webkit-keyframes pagelayer-rotate-1-in {
  0% {
    -webkit-transform: rotateX(180deg);
    opacity: 0;
  }
  35% {
    -webkit-transform: rotateX(120deg);
    opacity: 0;
  }
  65% {
    opacity: 0;
  }
  100% {
    -webkit-transform: rotateX(360deg);
    opacity: 1;
  }
}
@-moz-keyframes pagelayer-rotate-1-in {
  0% {
    -moz-transform: rotateX(180deg);
    opacity: 0;
  }
  35% {
    -moz-transform: rotateX(120deg);
    opacity: 0;
  }
  65% {
    opacity: 0;
  }
  100% {
    -moz-transform: rotateX(360deg);
    opacity: 1;
  }
}
@keyframes pagelayer-rotate-1-in {
  0% {
    -webkit-transform: rotateX(180deg);
    -moz-transform: rotateX(180deg);
    -ms-transform: rotateX(180deg);
    -o-transform: rotateX(180deg);
    transform: rotateX(180deg);
    opacity: 0;
  }
  35% {
    -webkit-transform: rotateX(120deg);
    -moz-transform: rotateX(120deg);
    -ms-transform: rotateX(120deg);
    -o-transform: rotateX(120deg);
    transform: rotateX(120deg);
    opacity: 0;
  }
  65% {
    opacity: 0;
  }
  100% {
    -webkit-transform: rotateX(360deg);
    -moz-transform: rotateX(360deg);
    -ms-transform: rotateX(360deg);
    -o-transform: rotateX(360deg);
    transform: rotateX(360deg);
    opacity: 1;
  }
}
@-webkit-keyframes pagelayer-rotate-1-out {
  0% {
    -webkit-transform: rotateX(0deg);
    opacity: 1;
  }
  35% {
    -webkit-transform: rotateX(-40deg);
    opacity: 1;
  }
  65% {
    opacity: 0;
  }
  100% {
    -webkit-transform: rotateX(180deg);
    opacity: 0;
  }
}
@-moz-keyframes pagelayer-rotate-1-out {
  0% {
    -moz-transform: rotateX(0deg);
    opacity: 1;
  }
  35% {
    -moz-transform: rotateX(-40deg);
    opacity: 1;
  }
  65% {
    opacity: 0;
  }
  100% {
    -moz-transform: rotateX(180deg);
    opacity: 0;
  }
}
@keyframes pagelayer-rotate-1-out {
  0% {
    -webkit-transform: rotateX(0deg);
    -moz-transform: rotateX(0deg);
    -ms-transform: rotateX(0deg);
    -o-transform: rotateX(0deg);
    transform: rotateX(0deg);
    opacity: 1;
  }
  35% {
    -webkit-transform: rotateX(-40deg);
    -moz-transform: rotateX(-40deg);
    -ms-transform: rotateX(-40deg);
    -o-transform: rotateX(-40deg);
    transform: rotateX(-40deg);
    opacity: 1;
  }
  65% {
    opacity: 0;
  }
  100% {
    -webkit-transform: rotateX(180deg);
    -moz-transform: rotateX(180deg);
    -ms-transform: rotateX(180deg);
    -o-transform: rotateX(180deg);
    transform: rotateX(180deg);
    opacity: 0;
  }
}

/*** xrotate-2 ***/
.pagelayer-aheading-rotate2 .pagelayer-words-wrapper {
  -webkit-perspective: 300px;
  -moz-perspective: 300px;
  perspective: 300px;
}
.pagelayer-aheading-rotate2 strong, .pagelayer-aheading-rotate2 b {
  display: inline-block;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
}
.pagelayer-aheading-rotate2 span {
  opacity: 0;
}
.pagelayer-aheading-rotate2 strong {
  -webkit-transform-style: preserve-3d;
  -moz-transform-style: preserve-3d;
  -ms-transform-style: preserve-3d;
  -o-transform-style: preserve-3d;
  transform-style: preserve-3d;
  -webkit-transform: translateZ(-20px) rotateX(90deg);
  -moz-transform: translateZ(-20px) rotateX(90deg);
  -ms-transform: translateZ(-20px) rotateX(90deg);
  -o-transform: translateZ(-20px) rotateX(90deg);
  transform: translateZ(-20px) rotateX(90deg);
  opacity: 0;
}
.pagelayer-is-visible .pagelayer-aheading-rotate2 strong {
  opacity: 1;
}
.pagelayer-aheading-rotate2 strong.pagelayer-aheading-in {
  -webkit-animation: pagelayer-rotate-2-in 0.4s forwards;
  -moz-animation: pagelayer-rotate-2-in 0.4s forwards;
  animation: pagelayer-rotate-2-in 0.4s forwards;
  -webkit-background-clip: text;
}
.pagelayer-aheading-rotate2 strong.pagelayer-aheading-out {
  -webkit-animation: pagelayer-rotate-2-out 0.4s forwards;
  -moz-animation: pagelayer-rotate-2-out 0.4s forwards;
  animation: pagelayer-rotate-2-out 0.4s forwards;
  -webkit-background-clip: text;
}
.pagelayer-aheading-rotate2 b {
  -webkit-transform: translateZ(20px);
  -moz-transform: translateZ(20px);
  -ms-transform: translateZ(20px);
  -o-transform: translateZ(20px);
  transform: translateZ(20px);
  -webkit-text-fill-color: transparent;
}

.pagelayer-no-csstransitions .pagelayer-aheading-rotate2 strong {
  -webkit-transform: rotateX(0deg);
  -moz-transform: rotateX(0deg);
  -ms-transform: rotateX(0deg);
  -o-transform: rotateX(0deg);
  transform: rotateX(0deg);
  opacity: 0;
}
.pagelayer-no-csstransitions .pagelayer-aheading-rotate2 strong b {
  -webkit-transform: scale(1);
  -moz-transform: scale(1);
  -ms-transform: scale(1);
  -o-transform: scale(1);
  transform: scale(1);
}

.pagelayer-no-csstransitions .pagelayer-aheading-rotate2 .pagelayer-is-visible strong {
  opacity: 1;
}

@-webkit-keyframes pagelayer-rotate-2-in {
  0% {
    opacity: 0;
    -webkit-transform: translateZ(-20px) rotateX(90deg);
  }
  60% {
    opacity: 1;
    -webkit-transform: translateZ(-20px) rotateX(-10deg);
  }
  100% {
    opacity: 1;
    -webkit-transform: translateZ(-20px) rotateX(0deg);
  }
}
@-moz-keyframes pagelayer-rotate-2-in {
  0% {
    opacity: 0;
    -moz-transform: translateZ(-20px) rotateX(90deg);
  }
  60% {
    opacity: 1;
    -moz-transform: translateZ(-20px) rotateX(-10deg);
  }
  100% {
    opacity: 1;
    -moz-transform: translateZ(-20px) rotateX(0deg);
  }
}
@keyframes pagelayer-rotate-2-in {
  0% {
    opacity: 0;
    -webkit-transform: translateZ(-20px) rotateX(90deg);
    -moz-transform: translateZ(-20px) rotateX(90deg);
    -ms-transform: translateZ(-20px) rotateX(90deg);
    -o-transform: translateZ(-20px) rotateX(90deg);
    transform: translateZ(-20px) rotateX(90deg);
  }
  60% {
    opacity: 1;
    -webkit-transform: translateZ(-20px) rotateX(-10deg);
    -moz-transform: translateZ(-20px) rotateX(-10deg);
    -ms-transform: translateZ(-20px) rotateX(-10deg);
    -o-transform: translateZ(-20px) rotateX(-10deg);
    transform: translateZ(-20px) rotateX(-10deg);
  }
  100% {
    opacity: 1;
    -webkit-transform: translateZ(-20px) rotateX(0deg);
    -moz-transform: translateZ(-20px) rotateX(0deg);
    -ms-transform: translateZ(-20px) rotateX(0deg);
    -o-transform: translateZ(-20px) rotateX(0deg);
    transform: translateZ(-20px) rotateX(0deg);
  }
}
@-webkit-keyframes pagelayer-rotate-2-out {
  0% {
    opacity: 1;
    -webkit-transform: translateZ(-20px) rotateX(0);
  }
  60% {
    opacity: 0;
    -webkit-transform: translateZ(-20px) rotateX(-100deg);
  }
  100% {
    opacity: 0;
    -webkit-transform: translateZ(-20px) rotateX(-90deg);
  }
}
@-moz-keyframes pagelayer-rotate-2-out {
  0% {
    opacity: 1;
    -moz-transform: translateZ(-20px) rotateX(0);
  }
  60% {
    opacity: 0;
    -moz-transform: translateZ(-20px) rotateX(-100deg);
  }
  100% {
    opacity: 0;
    -moz-transform: translateZ(-20px) rotateX(-90deg);
  }
}
@keyframes pagelayer-rotate-2-out {
  0% {
    opacity: 1;
    -webkit-transform: translateZ(-20px) rotateX(0);
    -moz-transform: translateZ(-20px) rotateX(0);
    -ms-transform: translateZ(-20px) rotateX(0);
    -o-transform: translateZ(-20px) rotateX(0);
    transform: translateZ(-20px) rotateX(0);
  }
  60% {
    opacity: 0;
    -webkit-transform: translateZ(-20px) rotateX(-100deg);
    -moz-transform: translateZ(-20px) rotateX(-100deg);
    -ms-transform: translateZ(-20px) rotateX(-100deg);
    -o-transform: translateZ(-20px) rotateX(-100deg);
    transform: translateZ(-20px) rotateX(-100deg);
  }
  100% {
    opacity: 0;
    -webkit-transform: translateZ(-20px) rotateX(-90deg);
    -moz-transform: translateZ(-20px) rotateX(-90deg);
    -ms-transform: translateZ(-20px) rotateX(-90deg);
    -o-transform: translateZ(-20px) rotateX(-90deg);
    transform: translateZ(-20px) rotateX(-90deg);
  }
}

/*** xloading-bar ***/
.pagelayer-aheading-loading-bar .pagelayer-words-wrapper {
  overflow: hidden;
  vertical-align: top;
}
.pagelayer-aheading-loading-bar .pagelayer-words-wrapper:after {
  content: '';
  position: absolute;
  left: 0;
  bottom: 0;
  height: 3px;
  width: 0;
  background: #0096a7;
  z-index: 2;
  -webkit-transition: width 0.3s -0.1s;
  -moz-transition: width 0.3s -0.1s;
  transition: width 0.3s -0.1s;
}
.pagelayer-aheading-loading-bar .pagelayer-words-wrapper.pagelayer-is-loading:after {
  width: 100%;
  -webkit-transition: width 3s;
  -moz-transition: width 3s;
  transition: width 3s;
}
.pagelayer-aheading-loading-bar span{
  top: .2em;
  opacity: 0;
  -webkit-transition: opacity 0.3s;
  -moz-transition: opacity 0.3s;
  transition: opacity 0.3s;
}
.pagelayer-aheading-loading-bar span.pagelayer-is-visible {
  opacity: 1;
  top: 0;
}

/*** xslide ***/
.pagelayer-aheading-slide .pagelayer-words-wrapper {
  overflow: hidden;
  vertical-align: top;
}
.pagelayer-aheading-slide span {
  opacity: 0;
  top: .2em;
}
.pagelayer-aheading-slide span.pagelayer-is-visible {
  top: 0;
  opacity: 1;
  -webkit-animation: pagelayer-slide-in 0.6s;
  -moz-animation: pagelayer-slide-in 0.6s;
  animation: pagelayer-slide-in 0.6s;
}
.pagelayer-aheading-slide span.pagelayer-is-hidden {
  -webkit-animation: pagelayer-slide-out 0.6s;
  -moz-animation: pagelayer-slide-out 0.6s;
  animation: pagelayer-slide-out 0.6s;
}

@-webkit-keyframes pagelayer-slide-in {
  0% {
    opacity: 0;
    -webkit-transform: translateY(-100%);
  }
  60% {
    opacity: 1;
    -webkit-transform: translateY(20%);
  }
  100% {
    opacity: 1;
    -webkit-transform: translateY(0);
  }
}
@-moz-keyframes pagelayer-slide-in {
  0% {
    opacity: 0;
    -moz-transform: translateY(-100%);
  }
  60% {
    opacity: 1;
    -moz-transform: translateY(20%);
  }
  100% {
    opacity: 1;
    -moz-transform: translateY(0);
  }
}
@keyframes pagelayer-slide-in {
  0% {
    opacity: 0;
    -webkit-transform: translateY(-100%);
    -moz-transform: translateY(-100%);
    -ms-transform: translateY(-100%);
    -o-transform: translateY(-100%);
    transform: translateY(-100%);
  }
  60% {
    opacity: 1;
    -webkit-transform: translateY(20%);
    -moz-transform: translateY(20%);
    -ms-transform: translateY(20%);
    -o-transform: translateY(20%);
    transform: translateY(20%);
  }
  100% {
    opacity: 1;
    -webkit-transform: translateY(0);
    -moz-transform: translateY(0);
    -ms-transform: translateY(0);
    -o-transform: translateY(0);
    transform: translateY(0);
  }
}
@-webkit-keyframes pagelayer-slide-out {
  0% {
    opacity: 1;
    -webkit-transform: translateY(0);
  }
  60% {
    opacity: 0;
    -webkit-transform: translateY(120%);
  }
  100% {
    opacity: 0;
    -webkit-transform: translateY(100%);
  }
}
@-moz-keyframes pagelayer-slide-out {
  0% {
    opacity: 1;
    -moz-transform: translateY(0);
  }
  60% {
    opacity: 0;
    -moz-transform: translateY(120%);
  }
  100% {
    opacity: 0;
    -moz-transform: translateY(100%);
  }
}
@keyframes pagelayer-slide-out {
  0% {
    opacity: 1;
    -webkit-transform: translateY(0);
    -moz-transform: translateY(0);
    -ms-transform: translateY(0);
    -o-transform: translateY(0);
    transform: translateY(0);
  }
  60% {
    opacity: 0;
    -webkit-transform: translateY(120%);
    -moz-transform: translateY(120%);
    -ms-transform: translateY(120%);
    -o-transform: translateY(120%);
    transform: translateY(120%);
  }
  100% {
    opacity: 0;
    -webkit-transform: translateY(100%);
    -moz-transform: translateY(100%);
    -ms-transform: translateY(100%);
    -o-transform: translateY(100%);
    transform: translateY(100%);
  }
}

/*** xclip ***/
.pagelayer-aheading-clip .pagelayer-words-wrapper {
  overflow: hidden;
  vertical-align: top;
}
.pagelayer-aheading-clip .pagelayer-words-wrapper:after {
  /* line */
  content: '';
  position: absolute;
  top: 0;
  right: 0;
  width: 3px;
  height: 100%;
  background-color: #aebcb9;
}
.pagelayer-aheading-clip span {
  opacity: 0;
}
.pagelayer-aheading-clip span.pagelayer-is-visible {
  opacity: 1;
}

/*** xzoom ***/
.pagelayer-aheading-zoom .pagelayer-words-wrapper {
  -webkit-perspective: 300px;
  -moz-perspective: 300px;
  perspective: 300px;
}
.pagelayer-aheading-zoom span{
  opacity: 0;
}
.pagelayer-aheading-zoom span.pagelayer-is-visible {
  opacity: 1;
  -webkit-animation: pagelayer-zoom-in 0.8s;
  -moz-animation: pagelayer-zoom-in 0.8s;
  animation: pagelayer-zoom-in 0.8s;
}
.pagelayer-aheading-zoom span.pagelayer-is-hidden {
  -webkit-animation: pagelayer-zoom-out 0.8s;
  -moz-animation: pagelayer-zoom-out 0.8s;
  animation: pagelayer-zoom-out 0.8s;
}

@-webkit-keyframes pagelayer-zoom-in {
  0% {
    opacity: 0;
    -webkit-transform: translateZ(100px);
  }
  100% {
    opacity: 1;
    -webkit-transform: translateZ(0);
  }
}
@-moz-keyframes pagelayer-zoom-in {
  0% {
    opacity: 0;
    -moz-transform: translateZ(100px);
  }
  100% {
    opacity: 1;
    -moz-transform: translateZ(0);
  }
}
@keyframes pagelayer-zoom-in {
  0% {
    opacity: 0;
    -webkit-transform: translateZ(100px);
    -moz-transform: translateZ(100px);
    -ms-transform: translateZ(100px);
    -o-transform: translateZ(100px);
    transform: translateZ(100px);
  }
  100% {
    opacity: 1;
    -webkit-transform: translateZ(0);
    -moz-transform: translateZ(0);
    -ms-transform: translateZ(0);
    -o-transform: translateZ(0);
    transform: translateZ(0);
  }
}
@-webkit-keyframes pagelayer-zoom-out {
  0% {
    opacity: 1;
    -webkit-transform: translateZ(0);
  }
  100% {
    opacity: 0;
    -webkit-transform: translateZ(-100px);
  }
}
@-moz-keyframes pagelayer-zoom-out {
  0% {
    opacity: 1;
    -moz-transform: translateZ(0);
  }
  100% {
    opacity: 0;
    -moz-transform: translateZ(-100px);
  }
}
@keyframes pagelayer-zoom-out {
  0% {
    opacity: 1;
    -webkit-transform: translateZ(0);
    -moz-transform: translateZ(0);
    -ms-transform: translateZ(0);
    -o-transform: translateZ(0);
    transform: translateZ(0);
  }
  100% {
    opacity: 0;
    -webkit-transform: translateZ(-100px);
    -moz-transform: translateZ(-100px);
    -ms-transform: translateZ(-100px);
    -o-transform: translateZ(-100px);
    transform: translateZ(-100px);
  }
}

/*** xrotate-3 ***/
.pagelayer-aheading-rotate3 .pagelayer-words-wrapper {
  -webkit-perspective: 300px;
  -moz-perspective: 300px;
  perspective: 300px;
}
.pagelayer-aheading-rotate3 span {
  opacity: 0;
}
.pagelayer-aheading-rotate3 strong {
  display: inline-block;
  -webkit-transform: rotateY(180deg);
  -moz-transform: rotateY(180deg);
  -ms-transform: rotateY(180deg);
  -o-transform: rotateY(180deg);
  transform: rotateY(180deg);
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
}
.pagelayer-is-visible .pagelayer-aheading-rotate3 strong {
  -webkit-transform: rotateY(0deg);
  -moz-transform: rotateY(0deg);
  -ms-transform: rotateY(0deg);
  -o-transform: rotateY(0deg);
  transform: rotateY(0deg);
}
.pagelayer-aheading-rotate3 strong.pagelayer-aheading-in {
  -webkit-animation: pagelayer-rotate-3-in 0.6s forwards;
  -moz-animation: pagelayer-rotate-3-in 0.6s forwards;
  animation: pagelayer-rotate-3-in 0.6s forwards;
}
.pagelayer-aheading-rotate3 strong.pagelayer-aheading-out {
  -webkit-animation: pagelayer-rotate-3-out 0.6s forwards;
  -moz-animation: pagelayer-rotate-3-out 0.6s forwards;
  animation: pagelayer-rotate-3-out 0.6s forwards;
}

.pagelayer-no-csstransitions .pagelayer-aheading-rotate3 strong {
  -webkit-transform: rotateY(0deg);
  -moz-transform: rotateY(0deg);
  -ms-transform: rotateY(0deg);
  -o-transform: rotateY(0deg);
  transform: rotateY(0deg);
  opacity: 0;
}

.pagelayer-no-csstransitions .pagelayer-aheading-rotate3 .pagelayer-is-visible strong {
  opacity: 1;
}

@-webkit-keyframes pagelayer-rotate-3-in {
  0% {
    -webkit-transform: rotateY(180deg);
  }
  100% {
    -webkit-transform: rotateY(0deg);
  }
}
@-moz-keyframes pagelayer-rotate-3-in {
  0% {
    -moz-transform: rotateY(180deg);
  }
  100% {
    -moz-transform: rotateY(0deg);
  }
}
@keyframes pagelayer-rotate-3-in {
  0% {
    -webkit-transform: rotateY(180deg);
    -moz-transform: rotateY(180deg);
    -ms-transform: rotateY(180deg);
    -o-transform: rotateY(180deg);
    transform: rotateY(180deg);
  }
  100% {
    -webkit-transform: rotateY(0deg);
    -moz-transform: rotateY(0deg);
    -ms-transform: rotateY(0deg);
    -o-transform: rotateY(0deg);
    transform: rotateY(0deg);
  }
}
@-webkit-keyframes pagelayer-rotate-3-out {
  0% {
    -webkit-transform: rotateY(0);
  }
  100% {
    -webkit-transform: rotateY(-180deg);
  }
}
@-moz-keyframes pagelayer-rotate-3-out {
  0% {
    -moz-transform: rotateY(0);
  }
  100% {
    -moz-transform: rotateY(-180deg);
  }
}
@keyframes pagelayer-rotate-3-out {
  0% {
    -webkit-transform: rotateY(0);
    -moz-transform: rotateY(0);
    -ms-transform: rotateY(0);
    -o-transform: rotateY(0);
    transform: rotateY(0);
  }
  100% {
    -webkit-transform: rotateY(-180deg);
    -moz-transform: rotateY(-180deg);
    -ms-transform: rotateY(-180deg);
    -o-transform: rotateY(-180deg);
    transform: rotateY(-180deg);
  }
}

/*** xscale ***/
.pagelayer-aheading-scale span {
  opacity: 0;
}
.pagelayer-aheading-scale strong {
  display: inline-block;
  opacity: 0;
  -webkit-transform: scale(0);
  -moz-transform: scale(0);
  -ms-transform: scale(0);
  -o-transform: scale(0);
  transform: scale(0);
}
.pagelayer-is-visible .pagelayer-aheading-scale strong {
  opacity: 1;
}
.pagelayer-aheading-scale strong.pagelayer-aheading-in {
  -webkit-animation: pagelayer-scale-up 0.6s forwards;
  -moz-animation: pagelayer-scale-up 0.6s forwards;
  animation: pagelayer-scale-up 0.6s forwards;
}
.pagelayer-aheading-scale strong.pagelayer-aheading-out {
  -webkit-animation: pagelayer-scale-down 0.6s forwards;
  -moz-animation: pagelayer-scale-down 0.6s forwards;
  animation: pagelayer-scale-down 0.6s forwards;
}

.pagelayer-no-csstransitions .pagelayer-aheading-scale strong {
  -webkit-transform: scale(1);
  -moz-transform: scale(1);
  -ms-transform: scale(1);
  -o-transform: scale(1);
  transform: scale(1);
  opacity: 0;
}

.pagelayer-no-csstransitions .pagelayer-aheading-scale .pagelayer-is-visible strong {
  opacity: 1;
}

@-webkit-keyframes pagelayer-scale-up {
  0% {
    -webkit-transform: scale(0);
    opacity: 0;
  }
  60% {
    -webkit-transform: scale(1.2);
    opacity: 1;
  }
  100% {
    -webkit-transform: scale(1);
    opacity: 1;
  }
}
@-moz-keyframes pagelayer-scale-up {
  0% {
    -moz-transform: scale(0);
    opacity: 0;
  }
  60% {
    -moz-transform: scale(1.2);
    opacity: 1;
  }
  100% {
    -moz-transform: scale(1);
    opacity: 1;
  }
}
@keyframes pagelayer-scale-up {
  0% {
    -webkit-transform: scale(0);
    -moz-transform: scale(0);
    -ms-transform: scale(0);
    -o-transform: scale(0);
    transform: scale(0);
    opacity: 0;
  }
  60% {
    -webkit-transform: scale(1.2);
    -moz-transform: scale(1.2);
    -ms-transform: scale(1.2);
    -o-transform: scale(1.2);
    transform: scale(1.2);
    opacity: 1;
  }
  100% {
    -webkit-transform: scale(1);
    -moz-transform: scale(1);
    -ms-transform: scale(1);
    -o-transform: scale(1);
    transform: scale(1);
    opacity: 1;
  }
}
@-webkit-keyframes pagelayer-scale-down {
  0% {
    -webkit-transform: scale(1);
    opacity: 1;
  }
  60% {
    -webkit-transform: scale(0);
    opacity: 0;
  }
}
@-moz-keyframes pagelayer-scale-down {
  0% {
    -moz-transform: scale(1);
    opacity: 1;
  }
  60% {
    -moz-transform: scale(0);
    opacity: 0;
  }
}
@keyframes pagelayer-scale-down {
  0% {
    -webkit-transform: scale(1);
    -moz-transform: scale(1);
    -ms-transform: scale(1);
    -o-transform: scale(1);
    transform: scale(1);
    opacity: 1;
  }
  60% {
    -webkit-transform: scale(0);
    -moz-transform: scale(0);
    -ms-transform: scale(0);
    -o-transform: scale(0);
    transform: scale(0);
    opacity: 0;
  }
}

/*** xpush ***/
.pagelayer-aheading-push span {
  opacity: 0;
}
.pagelayer-aheading-push span.pagelayer-is-visible {
  opacity: 1;
  -webkit-animation: pagelayer-push-in 0.6s;
  -moz-animation: pagelayer-push-in 0.6s;
  animation: pagelayer-push-in 0.6s;
}
.pagelayer-aheading-push span.pagelayer-is-hidden {
  -webkit-animation: pagelayer-push-out 0.6s;
  -moz-animation: pagelayer-push-out 0.6s;
  animation: pagelayer-push-out 0.6s;
}

@-webkit-keyframes pagelayer-push-in {
  0% {
    opacity: 0;
    -webkit-transform: translateX(-100%);
  }
  60% {
    opacity: 1;
    -webkit-transform: translateX(10%);
  }
  100% {
    opacity: 1;
    -webkit-transform: translateX(0);
  }
}
@-moz-keyframes pagelayer-push-in {
  0% {
    opacity: 0;
    -moz-transform: translateX(-100%);
  }
  60% {
    opacity: 1;
    -moz-transform: translateX(10%);
  }
  100% {
    opacity: 1;
    -moz-transform: translateX(0);
  }
}
@keyframes pagelayer-push-in {
  0% {
    opacity: 0;
    -webkit-transform: translateX(-100%);
    -moz-transform: translateX(-100%);
    -ms-transform: translateX(-100%);
    -o-transform: translateX(-100%);
    transform: translateX(-100%);
  }
  60% {
    opacity: 1;
    -webkit-transform: translateX(10%);
    -moz-transform: translateX(10%);
    -ms-transform: translateX(10%);
    -o-transform: translateX(10%);
    transform: translateX(10%);
  }
  100% {
    opacity: 1;
    -webkit-transform: translateX(0);
    -moz-transform: translateX(0);
    -ms-transform: translateX(0);
    -o-transform: translateX(0);
    transform: translateX(0);
  }
}
@-webkit-keyframes pagelayer-push-out {
  0% {
    opacity: 1;
    -webkit-transform: translateX(0);
  }
  60% {
    opacity: 0;
    -webkit-transform: translateX(110%);
  }
  100% {
    opacity: 0;
    -webkit-transform: translateX(100%);
  }
}
@-moz-keyframes pagelayer-push-out {
  0% {
    opacity: 1;
    -moz-transform: translateX(0);
  }
  60% {
    opacity: 0;
    -moz-transform: translateX(110%);
  }
  100% {
    opacity: 0;
    -moz-transform: translateX(100%);
  }
}
@keyframes pagelayer-push-out {
  0% {
    opacity: 1;
    -webkit-transform: translateX(0);
    -moz-transform: translateX(0);
    -ms-transform: translateX(0);
    -o-transform: translateX(0);
    transform: translateX(0);
  }
  60% {
    opacity: 0;
    -webkit-transform: translateX(110%);
    -moz-transform: translateX(110%);
    -ms-transform: translateX(110%);
    -o-transform: translateX(110%);
    transform: translateX(110%);
  }
  100% {
    opacity: 0;
    -webkit-transform: translateX(100%);
    -moz-transform: translateX(100%);
    -ms-transform: translateX(100%);
    -o-transform: translateX(100%);
    transform: translateX(100%);
  }
}

/* Animated Heading End */
/* Page Break Start */

.pagelayer-page-links{
text-align: center;
margin-top: 50px;
}

.pagelayer-page-links .post-page-numbers {
padding: 0;
margin: 0 0 .3em .3em;
border: 1px solid;
color: #000;
background: 0 0;
font-size: .8em;
width: 2.5em;
height: 2.5em;
line-height: calc(2.5em - 4px);
display: inline-block;
text-align: center;
transition: all .2s linear;
}

.pagelayer-arc-layout-left .pagelayer-wposts-col .pagelayer-wposts-content,
.pagelayer-arc-layout-right .pagelayer-wposts-col .pagelayer-wposts-featured,
.pagelayer-arc-layout-alt .pagelayer-wposts-col:nth-of-type(2n+1) .pagelayer-wposts-content,
.pagelayer-arc-layout-alt .pagelayer-wposts-col:nth-of-type(2n) .pagelayer-wposts-featured{
width: 48%;
float: left;
}

.pagelayer-arc-layout-right .pagelayer-wposts-col .pagelayer-wposts-content,
.pagelayer-arc-layout-left .pagelayer-wposts-col .pagelayer-wposts-featured,
.pagelayer-arc-layout-alt .pagelayer-wposts-col:nth-of-type(2n) .pagelayer-wposts-content,
.pagelayer-arc-layout-alt .pagelayer-wposts-col:nth-of-type(2n+1) .pagelayer-wposts-featured{
width: 48%;
float: right;
}
  
.pagelayer-arc-layout-left .pagelayer-wposts-thumb,
.pagelayer-arc-layout-right .pagelayer-wposts-thumb,
.pagelayer-arc-layout-alt .pagelayer-wposts-thumb{
display: block;
}
    
.pagelayer-wposts-thumb,
.pagelayer-wposts-featured,
.pagelayer-wposts-col{
overflow: hidden;
}

.pagelayer-loader-holder{
width: auto;
height: auto;
display: none;
}

.pagelayer-post-max,
.pagelayer-infinite-scroll-auto ~ .pagelayer_load_button .pagelayer-btn-load{
display: none;
}
/* Page Break End */

/********************/
/*** Freemium End ***/
/********************/