<?php
if (!defined( '__JETBACKUP__')) die('Direct access is not allowed');
class JetConfigDefines {
	const MAIN_DATA_FOLDER = 'jetbackup-1d0eab43aa08';
	const BACKUPS_FOLDER = 'backups-e0543c1f7015';
	const BACKUPS_TEMP_FOLDER = 'temp-417195451baf';
	const DB_MAIN_FOLDER = 'db-aa8c9b235cfe';
	const LOGS_FOLDER = 'logs-2cc67621a16f';
	const DOWNLOADS_FOLDER = 'downloads-5847560010b5';
	const ENCRYPTION_KEY = '92eabb6b6097';
	const UNIQUE_ID = '67222329a28f';
	const CRON_TOKEN = 'b9e5043df352';
	const SGB_LEGACY_CONVERTED = true;
}