<?php
/*
*
* JetBackup @ package
* Created By Idan Ben-Ezra
*
* Copyrights @ JetApps
* https://www.jetapps.com
*
**/
namespace JetBackup\Web\File;

use JetBackup\Exception\JBException;

defined( '__JETBACKUP__' ) or die( 'Restricted access' );

class FileException extends JBException {
}