/*!*****************************************************

Freak Flags, Copyright ©2026 Michael P. Cohen. Freak flags is licensed under the MIT license.  

For complete information visit: www.freakflagsprite.com 

******************************************************/

.lzflag {
    background-image:url(../images/flags.png);
    background-repeat:no-repeat;
    background-size: 100% 49494%;
    display: inline-block;
    overflow: hidden;
    position: relative;
    vertical-align: middle;
    box-sizing: content-box;
}
.lzflag-DZ {background-position:center 0.2287%}
.lzflag-AO {background-position:center 0.4524%}
.lzflag-BJ {background-position:center 0.6721%}
.lzflag-BW {background-position:center 0.8958%}
.lzflag-BF {background-position:center 1.1162%}
.lzflag-BI {background-position:center 1.3379%}
.lzflag-CM {background-position:center 1.5589%}
.lzflag-CV {background-position:center 1.7805%}
.lzflag-CF {background-position:center 2.0047%}
.lzflag-TD {background-position:center 2.2247%}
.lzflag-CD {background-position:left 2.4467%}
.lzflag-DJ {background-position:left 2.6674%}
.lzflag-EG {background-position:center 2.8931%}
.lzflag-GQ {background-position:center 3.1125%}
.lzflag-ER {background-position:left 3.3325%}
.lzflag-ET {background-position:center 3.5542%}
.lzflag-GA {background-position:center 3.7759%}
.lzflag-GM {background-position:center 4.0015%}
.lzflag-GH {background-position:center 4.2229%}
.lzflag-GN {background-position:center 4.441%}
.lzflag-GW {background-position:left 4.66663%}
.lzflag-CI {background-position:center 4.8844%}
.lzflag-KE {background-position:center 5.1061%}
.lzflag-LS {background-position:center 5.3298%}
.lzflag-LR {background-position:left 5.5495%}
.lzflag-LY {background-position:center 5.7712%}
.lzflag-MG {background-position:center 5.994%}
.lzflag-MW {background-position:center 6.2156%}
.lzflag-ML {background-position:center 6.4363%}
.lzflag-MR {background-position:center 6.658%}
.lzflag-MU {background-position:center 6.8805%}
.lzflag-YT {background-position:center 7.1038%}
.lzflag-MA {background-position:center 7.3231%}
.lzflag-MZ {background-position:left 7.5448%}
.lzflag-NA {background-position:left 7.7661%}
.lzflag-NE {background-position:center 7.98937%}
.lzflag-NG {background-position:center 8.2099%}
.lzflag-CG {background-position:center 8.4316%}
.lzflag-RE {background-position:center 8.6533%}
.lzflag-RW {background-position:right 8.875%}
.lzflag-SH {background-position:center 9.0967%}
.lzflag-ST {background-position:center 9.322%}
.lzflag-SN {background-position:center 9.5426%}
.lzflag-SC {background-position:left 9.7628%}
.lzflag-SL {background-position:center 9.9845%}
.lzflag-SO {background-position:center 10.2052%}
.lzflag-ZA {background-position:left 10.4269%}
.lzflag-SS {background-position:left 10.6486%}
.lzflag-SD {background-position:center 10.8703%}
.lzflag-SR {background-position:center 11.0945%}
.lzflag-SZ {background-position:center 11.3135%}
.lzflag-TG {background-position:left 11.5354%}
.lzflag-TN {background-position:center 11.7593%}
.lzflag-UG {background-position:center 11.9799%}
.lzflag-TZ {background-position:center 12.2005%}
.lzflag-EH {background-position:center 12.4222%}
.lzflag-YE {background-position:center 12.644%}
.lzflag-ZM {background-position:center 12.8664%}
.lzflag-ZW {background-position:left 13.0873%}
.lzflag-AI {background-position:center 13.309%}
.lzflag-AG {background-position:center 13.5307%}
.lzflag-AR {background-position:center 13.7524%}
.lzflag-AW {background-position:left 13.9741%}
.lzflag-BS {background-position:left 14.1958%}
.lzflag-BB {background-position:center 14.4175%}
.lzflag-BQ {background-position:center 14.6415%}
.lzflag-BZ {background-position:center 14.8609%}
.lzflag-BM {background-position:center 15.0826%}
.lzflag-BO {background-position:center 15.306%}
.lzflag-VG {background-position:center 15.528%}
.lzflag-BR {background-position:center 15.7496%}
.lzflag-CA {background-position:center 15.9694%}
.lzflag-KY {background-position:center 16.1911%}
.lzflag-CL {background-position:left 16.4128%}
.lzflag-CO {background-position:left 16.6345%}
.lzflag-KM {background-position:center 16.8562%}
.lzflag-CR {background-position:center 17.0779%}
.lzflag-CU {background-position:left 17.2996%}
.lzflag-CW {background-position:center 17.5213%}
.lzflag-DM {background-position:center 17.743%}
.lzflag-DO {background-position:center 17.968%}
.lzflag-EC {background-position:center 18.1864%}
.lzflag-SV {background-position:center 18.411%}
.lzflag-FK {background-position:center 18.6298%}
.lzflag-GF {background-position:center 18.8515%}
.lzflag-GL {background-position:left 19.0732%}
.lzflag-GD {background-position:center 19.2987%}
.lzflag-GP {background-position:center 19.518%}
.lzflag-GT {background-position:center 19.7383%}
.lzflag-GY {background-position:center 19.96%}
.lzflag-HT {background-position:center 20.1817%}
.lzflag-HN {background-position:center 20.4034%}
.lzflag-JM {background-position:center 20.6241%}
.lzflag-MQ {background-position:center 20.8468%}
.lzflag-MX {background-position:center 21.0685%}
.lzflag-MS {background-position:center 21.2902%}
.lzflag-NI {background-position:center 21.5119%}
.lzflag-PA {background-position:center 21.7336%}
.lzflag-PY {background-position:center 21.9553%}
.lzflag-PE {background-position:center 22.177%}
.lzflag-PR {background-position:left 22.4002%}
.lzflag-BL {background-position:center 22.6204%}
.lzflag-KN {background-position:center 22.8421%}
.lzflag-LC {background-position:center 23.0638%}
.lzflag-PM {background-position:center 23.2855%}
.lzflag-VC {background-position:center 23.5072%}
.lzflag-SX {background-position:left 23.732%}
.lzflag-TT {background-position:center 23.9506%}
.lzflag-TC {background-position:center 24.1723%}
.lzflag-US {background-position:center 24.394%}
.lzflag-VI {background-position:center 24.6157%}
.lzflag-UY {background-position:left 24.8374%}
.lzflag-VE {background-position:center 25.0591%}
.lzflag-AB {background-position:center 25.2808%}
.lzflag-AF {background-position:center 25.5025%}
.lzflag-AZ {background-position:center 25.7242%}
.lzflag-BD {background-position:center 25.9459%}
.lzflag-BT {background-position:center 26.1676%}
.lzflag-BN {background-position:center 26.3885%}
.lzflag-KH {background-position:center 26.611%}
.lzflag-CN {background-position:left 26.8327%}
.lzflag-GE {background-position:center 27.0544%}
.lzflag-HK {background-position:center 27.2761%}
.lzflag-IN {background-position:center 27.4978%}
.lzflag-ID {background-position:center 27.7195%}
.lzflag-JP {background-position:center 27.9412%}
.lzflag-KZ {background-position:center 28.1615%}
.lzflag-LA {background-position:center 28.3846%}
.lzflag-MO {background-position:center 28.6063%}
.lzflag-MY {background-position:center 28.829%}
.lzflag-MV {background-position:center 29.0497%}
.lzflag-MN {background-position:left 29.2714%}
.lzflag-MM {background-position:center 29.4931%}
.lzflag-NP {background-position:left 29.7148%}
.lzflag-KP {background-position:left 29.9365%}
.lzflag-MP {background-position:center 30.1582%}
.lzflag-PW {background-position:center 30.3799%}
.lzflag-PG {background-position:center 30.6016%}
.lzflag-PH {background-position:left 30.8233%}
.lzflag-SG {background-position:left 31.045%}
.lzflag-KR {background-position:center 31.2667%}
.lzflag-LK {background-position:right 31.4884%}
.lzflag-TW {background-position:left 31.7101%}
.lzflag-TJ {background-position:center 31.9318%}
.lzflag-TH {background-position:center 32.1535%}
.lzflag-TL {background-position:left 32.3752%}
.lzflag-TM {background-position:center 32.5969%}
.lzflag-VN {background-position:center 32.8186%}
.lzflag-AX {background-position:center 33.0403%}
.lzflag-AL {background-position:center 33.25975%}
.lzflag-AD {background-position:center 33.4837%}
.lzflag-AM {background-position:center 33.7054%}
.lzflag-AT {background-position:center 33.9271%}
.lzflag-BY {background-position:left 34.1488%}
.lzflag-BE {background-position:center 34.3705%}
.lzflag-BA {background-position:center 34.5922%}
.lzflag-BG {background-position:center 34.8139%}
.lzflag-HR {background-position:center 35.0356%}
.lzflag-CY {background-position:center 35.2555%}
.lzflag-CZ {background-position:left 35.479%}
.lzflag-DK {background-position:center 35.7007%}
.lzflag-EE {background-position:center 35.9224%}
.lzflag-FO {background-position:center 36.1441%}
.lzflag-FI {background-position:center 36.3658%}
.lzflag-FR {background-position:center 36.5875%}
.lzflag-DE {background-position:center 36.8092%}
.lzflag-GI {background-position:center 37.0309%}
.lzflag-GR {background-position:left 37.2526%}
.lzflag-GG {background-position:center 37.4743%}
.lzflag-HU {background-position:center 37.696%}
.lzflag-IS {background-position:center 37.9177%}
.lzflag-IE {background-position:center 38.1394%}
.lzflag-IM {background-position:center 38.3611%}
.lzflag-IT {background-position:center 38.5828%}
.lzflag-JE {background-position:center 38.8045%}
.lzflag-XK {background-position:center 39.0262%}
.lzflag-LV {background-position:center 39.2479%}
.lzflag-LI {background-position:left 39.4696%}
.lzflag-LT {background-position:center 39.6913%}
.lzflag-LU {background-position:center 39.913%}
.lzflag-MT {background-position:left 40.1347%}
.lzflag-MD {background-position:center 40.3564%}
.lzflag-MC {background-position:center 40.5781%}
.lzflag-ME {background-position:center 40.7998%}
.lzflag-NL {background-position:center 41.0215%}
.lzflag-MK {background-position:center 41.2432%}
.lzflag-NO {background-position:center 41.4649%}
.lzflag-PL {background-position:center 41.6866%}
.lzflag-PT {background-position:center 41.9083%}
.lzflag-RO {background-position:center 42.13%}
.lzflag-RU {background-position:center 42.3517%}
.lzflag-SM {background-position:center 42.5734%}
.lzflag-RS {background-position:center 42.7951%}
.lzflag-SK {background-position:center 43.0168%}
.lzflag-SI {background-position:center 43.2385%}
.lzflag-ES {background-position:left 43.4602%}
.lzflag-SE {background-position:center 43.6819%}
.lzflag-CH {background-position:center 43.9036%}
.lzflag-TR {background-position:center 44.1253%}
.lzflag-UA {background-position:center 44.347%}
.lzflag-GB {background-position:center 44.5687%}
.lzflag-VA {background-position:right 44.7904%}
.lzflag-BH {background-position:center 45.0121%}
.lzflag-IR {background-position:center 45.2338%}
.lzflag-IQ {background-position:center 45.4555%}
.lzflag-IL {background-position:center 45.6772%}
.lzflag-KW {background-position:left 45.897%}
.lzflag-JO {background-position:left 46.1206%}
.lzflag-KG {background-position:center 46.3423%}
.lzflag-LB {background-position:center 46.561%}
.lzflag-OM {background-position:left 46.7857%}
.lzflag-PK {background-position:center 47.0074%}
.lzflag-PS {background-position:center 47.2291%}
.lzflag-QA {background-position:center 47.4508%}
.lzflag-SA {background-position:center 47.6725%}
.lzflag-SY {background-position:center 47.8942%}
.lzflag-AE {background-position:center 48.1159%}
.lzflag-UZ {background-position:left 48.3376%}
.lzflag-AS {background-position:right 48.5593%}
.lzflag-AU {background-position:center 48.781%}
.lzflag-CX {background-position:center 49.002%}
.lzflag-CC {background-position:center 49.2244%}
.lzflag-CK {background-position:center 49.4445%}
.lzflag-FJ {background-position:center 49.6678%}
.lzflag-PF {background-position:center 49.8895%}
.lzflag-GU {background-position:center 50.1112%}
.lzflag-KI {background-position:center 50.3329%}
.lzflag-MH {background-position:left 50.5546%}
.lzflag-FM {background-position:center 50.7763%}
.lzflag-NC {background-position:center 50.998%}
.lzflag-NZ {background-position:center 51.2197%}
.lzflag-NR {background-position:left 51.4414%}
.lzflag-NU {background-position:center 51.6631%}
.lzflag-NF {background-position:center 51.8848%}
.lzflag-WS {background-position:left 52.1065%}
.lzflag-SB {background-position:left 52.3282%}
.lzflag-TK {background-position:center 52.5499%}
.lzflag-TO {background-position:left 52.7716%}
.lzflag-TV {background-position:center 52.9933%}
.lzflag-VU {background-position:left 53.215%}
.lzflag-WF {background-position:center 53.4315%}

.lzflag.ff-sm {width: 18px;height: 11.5px}
.lzflag.ff-md {width: 27px;height: 17px}
.lzflag.ff-lg {width: 42px;height: 27px}
.lzflag.ff-xl {width: 60px;height: 37px}
.lzflag-CH.ff-sm {width: 11.5px;height: 11.5px}
.lzflag-CH.ff-md {width: 17px;height: 17px}
.lzflag-CH.ff-lg {width: 27px;height: 27px}
.lzflag-CH.ff-xl {width: 37px;height: 37px}
.lzflag-CH {background-size: 153% 49494%}
.lzflag-NP.ff-lt {background-color: rgb(220,220,220);clip-path: polygon(-5% -12%, 0 100%, 100% 119%, 49% 61%, 101% 61%)}
.lzflag-NP.ff-dk {background-color: rgb(90,90,90);clip-path: polygon(-5% -12%, 0 100%, 100% 119%, 49% 61%, 101% 61%)}
.lzflag-NP.ff-sm {width: 14px;height: 11.5px}
.lzflag-NP.ff-md {width: 20px;height: 17px}
.lzflag-NP.ff-lg {width: 31px;height: 27px}
.lzflag-NP.ff-xl {width: 42px;height: 37px}
.lzflag-NP {background-size: 136% 49494%}
.lzflag-NP {clip-path: polygon(-6% -9%, 0 100%, 86% 114%, 37% 56.5%, 86% 56.5%)}
.lzflag-NP.ff-orb,
.lzflag-NP.ff-sphere,
.lzflag-NP.ff-app,
.lzflag-NP.ff-leaf {clip-path: none;
background-color: transparent}
.lzflag-NP.ff-round {clip-path: polygon(-5% -11%, 0 100%, 100% 106%, 55% 61.5%, 115% 61.5%)}
