/*Donut Chart*/

.speedycache-test-notice {
border-left: none;
background-color:white;
color: #0F172A;
border-radius: 8px;
padding: 0px 15px;
box-shadow: 0 0 #0000, 0 0 #0000, 0 1px 3px 0 #0000001a, 0 1px 2px -1px #0000001a;
}

.speedycache-test-notice p{
margin:0px;
padding:0px;    
}

.speedycache-test-chart-wrap {
display: flex;
align-items: center;
justify-content: center;
flex-wrap: wrap;
}

.speedycache-notice-title {
text-align: center;
font-weight: 600;
font-size: 20px;
}

.speedycache-test-donut-text {
font-family: Arial, Helvetica, sans-serif;
fill: #FF6200;
}

.speedycache-donut-wrap {
width: 100%;
max-width: 170px;
font-size: 16px;
animation: donutfade 1s;
text-align: center;
}

.speedycache-donut-segment {
transform-origin: center;
}

.speedycache-donut-percent {
font-size: 0.85em;
line-height: 0.8;
transform: translateY(0.5em);
font-weight: bold;
}

.speedycache-donut-label {
font-size: 1em;
font-weight: 700;
line-height: 1;
transform: translateY(0.25em);
}

.speedycache-test-action{
margin:10px auto;
text-align:center;
font-weight:400;
line-height:2;
font-size:14px;
}

.speedycache-enable-btn{
background-color: #3d5afe;
border: none;
color: #FFF;
outline: none;
border-radius: 4px;
box-shadow: 1px 0 5px 0 #ccc;
padding: 8px 14px;
font-weight: bold;
font-size: 0.9rem;
cursor: pointer;
transition: background-color 0.4s, transform 0.3s;
}

.speedycache-enable-btn:hover{
background-color:#0529f7;
transform: translateY(-1px);
transition: background-color 0.6s, transform 0.4s;
color:#FFF;
}

.speedycache-img-stat-update-status{
display:none;
position:absolute;
bottom:5px;
right:10px;
display:flex;
align-items:center;
gap:5px;
font-size:12px;
color:#969696;
}

.speedycache-img-stat-update-status .dashicons{
font-size:12px;
width:12px;
height:12px;
animation: speedycache-pro-spin 2s linear infinite;
}

@keyframes speedycache-pro-spin {
0% {
transform: rotate(0deg);
}
100% {
transform: rotate(360deg);
}
}

/* Automatic Optimization cron */
.speedycache-cron-container {
padding: 20px 25px;
border: 1px solid #e1e1e1;
border-radius: 8px;
max-width: 100%;
margin-top: 15px;
}

.speedycache-cron-container h3 {
margin-bottom: 5px;
}

.speedycache-cron-container p {
margin-top: 0;
color: #555;
font-size: 14px;
}

.speedycache-cron-inside-checkbox{
display: grid;
grid-template-columns: 1fr 1fr 1fr;
margin:10px 0px;
gap: 10px;
}
