<div ng-controller="destinationBox">

    <div class="form-group row" ng-if="!saveData.options.refresh_token">
        <label for="access_code" class="col-sm-3 col-form-label jb-label">{{ lang.t("Access Code") }}</label>
        <div class="col-sm-7">
            <div class="input-group">
                <input id="access_code" required="required" class="form-control col-md-7 col-xs-12" type="text" ng-model="saveData.options.access_code" />
                <div class="input-group-btn"><a href="" class="btn btn-primary jb-background-orange border-0 ml-10" ng-click="getAuthCode()">{{ lang.t("Get Access Code") }}</a></div>
            </div>

        </div>
    </div>

    <div class="form-group row">
        <label for="box_retries" class="col-sm-3 col-form-label jb-label">{{ lang.t("HTTP Retries") }}</label>
        <div class="col-sm-7">
            <input-dropdown box_retries="Retries" options="{0:lang.t('Disabled'),1:1,3:3,5:5,10:10}" ng-model="saveData.options.retries">
                <input id="box_retries" validate-field="int" min="0" class="form-control" ng-model="saveData.options.retries" id="retries" type="text" value="" />
            </input-dropdown>
            <p class="pd-3">{{ lang.t("If the HTTP call (upload / download / delete) fails for some reason retry for x times. Maximum 10") }}</p>
        </div>
    </div>

</div>