<?php

declare(strict_types=1);

/**
 * Finite Fields Base Class
 *
 * PHP version 5 and 7
 *
 * @author    Jim Wigginton <terrafrost@php.net>
 * @copyright 2017 Jim Wigginton
 * @license   http://www.opensource.org/licenses/mit-license.html  MIT License
 */

namespace phpseclib3\Math\Common;

/**
 * Finite Fields
 *
 * @author  Jim Wigginton <terrafrost@php.net>
 */
abstract class FiniteField
{
}
